@@ -3,7 +3,7 @@
 **
 **	(c) COPYRIGHT MIT 1995.
 **	Please first read the full copyright statement in the file COPYRIGH.
-**	@(#) $Id: HTHost.c,v 2.65 1999/05/03 15:56:46 frystyk Exp $
+**	@(#) $Id: HTHost.c,v 2.70 2000/06/23 14:23:52 kahan Exp $
 **
 **	This object manages the information that we know about a remote host.
 **	This can for example be what type of host it is, and what version
@@ -154,10 +154,11 @@ PRIVATE int IdleTimeoutEvent (HTTimer * timer, void * param, HTEventType type)
 {
     HTHost * host = (HTHost *) param;
     SOCKET sockfd = HTChannel_socket(host->channel);
-    int result = HostEvent (sockfd, host, HTEvent_CLOSE);
+
     HTTimer_delete(timer);
     host->timer = NULL;
-    return result;
+
+    return HostEvent (sockfd, host, HTEvent_CLOSE);
 }
 
 /*
@@ -181,11 +182,11 @@ PRIVATE int HostEvent (SOCKET soc, void * pVoid, HTEventType type)
              * of previous socket has been dispensed by the library,
              * the section below makes sure the event does not get miss attributed
              */
-	    if (HTChannel_socket(host->channel) != soc) {
-		 HTTRACE(CORE_TRACE, "Host Event.. wild socket %d type = %s real socket is %d\n" _ soc _ 
-			 type == HTEvent_CLOSE ? "Event_Close" : "Event_Read" _ 
-			 HTChannel_socket(host->channel));
-		 return HT_OK;
+	    if (HTChannel_socket(host->channel) != soc && type != HTEvent_ACCEPT && !host->listening) {
+		HTTRACE(CORE_TRACE, "Host Event.. wild socket %d type = %s real socket is %d\n" _ soc _ 
+			type == HTEvent_CLOSE ? "Event_Close" : "Event_Read" _ 
+			HTChannel_socket(host->channel));
+		return HT_OK;
 	    }
 
 	    targetNet = (HTNet *)HTList_firstObject(host->pipeline);
@@ -220,11 +221,13 @@ PRIVATE int HostEvent (SOCKET soc, void * pVoid, HTEventType type)
 	    char buf[256];
 	    int ret;
 	    memset(buf, '\0', sizeof(buf));
-	    while ((ret = NETREAD(HTChannel_socket(host->channel), buf, sizeof(buf)-1)) > 0) {
+	    if (HTChannel_socket(host->channel) != INVSOC) {
+	      while ((ret = NETREAD(HTChannel_socket(host->channel), buf, sizeof(buf)-1)) > 0) {
 		HTTRACE(CORE_TRACE, "Host Event.. Host %p `%s\' had %d extraneous bytes: `%s\'\n" _ 
-			    host _ host->hostname _ ret _ buf);
+			host _ host->hostname _ ret _ buf);
 		memset(buf, '\0', sizeof(buf));		
-	    }	    
+	      }	    
+	    }
 	}
 	HTHost_clearChannel(host, HT_OK);
 	return HT_OK; 	     /* extra garbage does not constitute an application error */
@@ -363,7 +366,7 @@ PUBLIC HTHost * HTHost_newWParse (HTRequest * request, char * url, u_short u_por
 
     fullhost = HTParse(proxy ? proxy : url, "", PARSE_HOST);
 
-	      /* If there's an @ then use the stuff after it as a hostname */
+    /* If there's an @ then use the stuff after it as a hostname */
     if (fullhost) {
 	char * at_sign;
 	if ((at_sign = strchr(fullhost, '@')) != NULL)
@@ -790,6 +793,8 @@ PUBLIC BOOL HTHost_clearChannel (HTHost * host, int status)
 	host->broken_pipe = NO;
        	host->mode = HT_TP_SINGLE;
 
+	host->recovered = 0;
+
 	HTTRACE(CORE_TRACE, "Host info... removed host %p as persistent\n" _ host);
 
 	if (!HTList_isEmpty(host->pending)) {
@@ -1091,15 +1096,19 @@ PRIVATE BOOL HTHost_free (HTHost * host, int status)
 		    HTTRACE(CORE_TRACE, "%d requests made, %d in pipe, max %d requests pr connection\n" _ 
 				host->reqsMade _ piped _ host->reqsPerConnection);
 		    host->do_recover = YES;
-		    HTChannel_delete(host->channel, status);
-		    host->channel = NULL;
+		    /* @@ JK: not clear yet if I need or not to set the channel to NULL. I think not */
+		    /* HTChannel_delete(host->channel, status);  */
+		    if (HTChannel_delete(host->channel, status)) {
+			HTTRACE(CORE_TRACE, "Host Event.. clearing channel on host %p (%s)\n" _ host _ host->hostname);
+			host->channel = NULL;
+		    }
 		} else {
 		    HTChannel_setSemaphore(host->channel, 0);
 		    HTHost_clearChannel(host, status);
 		}
 	    } else if (piped<=1 && host->reqsMade==host->reqsPerConnection) {
                 HTTRACE(CORE_TRACE, "Host Object. closing persistent socket %d\n" _ 
-					HTChannel_socket(host->channel));
+			HTChannel_socket(host->channel));
                 
                 /* 
                 **  By lowering the semaphore we make sure that the channel
@@ -1110,7 +1119,7 @@ PRIVATE BOOL HTHost_free (HTHost * host, int status)
 
             } else {
                 HTTRACE(CORE_TRACE, "Host Object. keeping persistent socket %d\n" _ 
-					HTChannel_socket(host->channel));
+			HTChannel_socket(host->channel));
                 if (HTChannel_delete(host->channel, status)) {
 		    HTDEBUGBREAK("Host Event.. Channel unexpected deleted from host %p (%s)\n" _ host _ host->hostname);
 		    host->channel = NULL;
@@ -1278,7 +1287,7 @@ PUBLIC int HTHost_numberOfPendingNetObjects (HTHost * host)
 **	multiple pipelined engines. It then registers its own engine 
 **	(HostEvent) with the event manager.
 */
-PUBLIC int HTHost_connect (HTHost * host, HTNet * net, char * url, HTProtocolId port)
+PUBLIC int HTHost_connect (HTHost * host, HTNet * net, char * url)
 {
     HTRequest * request = HTNet_request(net);
     int status = HT_OK;
@@ -1303,7 +1312,7 @@ PUBLIC int HTHost_connect (HTHost * host, HTNet * net, char * url, HTProtocolId
     }
 
     if (!host->lock || (host->lock && host->lock == net)) {
-	status = HTDoConnect(net, url, port);
+	status = HTDoConnect(net);
 	if (status == HT_PENDING)
 	    return HT_WOULD_BLOCK;
 	else if (status == HT_WOULD_BLOCK) {
@@ -1318,7 +1327,7 @@ PUBLIC int HTHost_connect (HTHost * host, HTNet * net, char * url, HTProtocolId
 	    HTNet * next_pending = NULL;
 	    if ((next_pending = HTList_firstObject(host->pending))) {
 		HTTRACE(CORE_TRACE, "Host connect Changing lock on Host %p to %p\n" _ 
-					host _ next_pending);
+			host _ next_pending);
 		host->lock = next_pending;	    
 	    } else {
 		HTTRACE(CORE_TRACE, "Host connect Unlocking Host %p\n" _ host);
@@ -1335,8 +1344,7 @@ PUBLIC int HTHost_connect (HTHost * host, HTNet * net, char * url, HTProtocolId
     return HT_ERROR; /* @@@ - some more deletion and stuff here? */
 }
 
-PUBLIC int HTHost_accept (HTHost * host, HTNet * net, HTNet ** accepted,
-			  char * url, HTProtocolId port)
+PUBLIC int HTHost_listen (HTHost * host, HTNet * net, char * url)
 {
     HTRequest * request = HTNet_request(net);
     int status = HT_OK;
@@ -1344,52 +1352,75 @@ PUBLIC int HTHost_accept (HTHost * host, HTNet * net, HTNet ** accepted,
 	HTProtocol * protocol = HTNet_protocol(net);
 	if ((host = HTHost_newWParse(request, url, HTProtocol_id(protocol))) == NULL)
 	    return HT_ERROR;
-	else {
-	    SockA *sin = &host->sock_addr;
-	    sin->sin_addr.s_addr = INADDR_ANY;
-	}
 
 	/*
 	** If not already locked and without a channel
-	** then lock the darn thing
+	** then lock the darn thing with the first Net object
+	** pending.
 	*/
 	if (!host->lock && !host->channel) {
 	    host->forceWriteFlush = YES;
 	    host->lock = net;
 	}
 	HTNet_setHost(net, host);
+    }
 
-	/*
-	** Start listening on the socket
-	*/
-	{
-	    status = HTDoListen(net, port, INVSOC, HT_BACKLOG);
-	    if (status != HT_OK) {
-		HTTRACE(CORE_TRACE, "Listen...... On Host %p resulted in %d\n" _ host _ status);
-		return HT_ERROR;
-	    }
-	}
+    /*
+    ** See if we already have a dedicated listen Net object. If not 
+    ** then create one.
+    */
+    if (!host->listening) host->listening = HTNet_new(host);
+
+    /*
+    ** Start listening on the Net object
+    */
+    status = HTDoListen(host->listening, net, HT_BACKLOG);
+    if (status != HT_OK) {
+	HTTRACE(CORE_TRACE, "Host listen. On Host %p resulted in %d\n" _ host _ status);
+	return status;
+    }
+    return HT_OK;
+}
+
+PUBLIC int HTHost_accept (HTHost * host, HTNet * net, char * url)
+{
+    int status = HT_OK;
+    if (!host || !host->listening) {
+	HTTRACE(CORE_TRACE, "Host accept. No host object or not listening on anything\n");
+	return HT_ERROR;
     }
 
     if (!host->lock || (host->lock && host->lock == net)) {
-	status = HTDoAccept(net, accepted);
-	if (status == HT_OK) {
-
-	    /* Add the new accepted Net object to the pipeline */
-	    HTList_appendObject(host->pipeline, *accepted);
-
-	    /* Unlock the accept object */
-	    host->lock = NULL;
-
-	    return HT_OK;
-	}
-	if (status == HT_WOULD_BLOCK) {
+	status = HTDoAccept(host->listening, net);
+	if (status == HT_PENDING)
+	    return HT_WOULD_BLOCK;
+	else if (status == HT_WOULD_BLOCK) {
 	    host->lock = net;
 	    return status;
+	} else {
+
+	    /*
+	    **  See if there is already a new pending request that should
+	    **  take over the current lock
+	    */
+	    HTNet * next_pending = NULL;
+	    if ((next_pending = HTList_firstObject(host->pending))) {
+		HTTRACE(CORE_TRACE, "Host connect Changing lock on Host %p to %p\n" _ 
+			host _ next_pending);
+		host->lock = next_pending;	    
+	    } else {
+		HTTRACE(CORE_TRACE, "Host connect Unlocking Host %p\n" _ host);
+		host->lock = NULL;	    
+	    }
+	    return status;
+	}
+    } else {
+	HTTRACE(CORE_TRACE, "Host connect Host %p already locked with %p\n" _ host _ host->lock);
+	if ((status = HTHost_addNet(host, net)) == HT_PENDING) {
+	    return HT_PENDING;
 	}
-	if (status == HT_PENDING) return HT_WOULD_BLOCK;
     }
-    return HT_ERROR; /* @@@ - some more deletion and stuff here? */
+    return HT_ERROR;
 }
 
 /*
