@@ -20,6 +20,7 @@
  *
  * Authors: V. Quint (INRIA)
  *          C. Roisin (INRIA) - Columns and pages
+ *          R. GUetari (W3C/INRIA): Unicode routines.
  *
  */
 
@@ -46,8 +47,8 @@ int                 n;
 
 #endif /* __STDC__ */
 {
-   TtaWriteByte (outfile, (CHAR_T) (n / 256));
-   TtaWriteByte (outfile, (CHAR_T) (n % 256));
+   TtaWriteByte (outfile, (char) (n / 256));
+   TtaWriteByte (outfile, (char) (n % 256));
 }
 
 /*----------------------------------------------------------------------
@@ -82,8 +83,8 @@ Name                n;
 
    i = 0;
    do
-      TtaWriteByte (outfile, n[i++]);
-   while (n[i - 1] != '\0');
+      TtaWriteWideChar (outfile, n[i++], ISO_8859_1 /* UTF_8 */);
+   while (n[i - 1] != TEXT('\0'));
 }
 
 
@@ -151,6 +152,9 @@ TypeUnit            unit;
 	    case UnPercent:
 	       TtaWriteByte (outfile, C_UNIT_PERCENT);
 	       break;
+	    case UnAuto:
+	       TtaWriteByte (outfile, C_UNIT_AUTO);
+	       break;
 	    default:
 	       fprintf (stderr, "Invalid unit %X\n", (int)unit);
 	       TtaWriteByte (outfile, C_UNIT_REL);
@@ -195,6 +199,66 @@ PRuleType           ruleType;
 	    case PtHorizPos:
 	       TtaWriteByte (outfile, C_PR_HPOS);
 	       break;
+            case PtMarginTop:
+               TtaWriteByte (outfile, C_PR_MARGINTOP);
+               break;
+            case PtMarginRight:
+               TtaWriteByte (outfile, C_PR_MARGINRIGHT);
+               break;
+            case PtMarginBottom:
+               TtaWriteByte (outfile, C_PR_MARGINBOTTOM);
+               break;
+            case PtMarginLeft:
+               TtaWriteByte (outfile, C_PR_MARGINLEFT);
+               break;
+            case PtPaddingTop:
+               TtaWriteByte (outfile, C_PR_PADDINGTOP);
+               break;
+            case PtPaddingRight:
+               TtaWriteByte (outfile, C_PR_PADDINGRIGHT);
+               break;
+            case PtPaddingBottom:
+               TtaWriteByte (outfile, C_PR_PADDINGBOTTOM);
+               break;
+            case PtPaddingLeft:
+               TtaWriteByte (outfile, C_PR_PADDINGLEFT);
+               break;
+            case PtBorderTopWidth:
+               TtaWriteByte (outfile, C_PR_BORDERTOPWIDTH);
+               break;
+            case PtBorderRightWidth:
+               TtaWriteByte (outfile, C_PR_BORDERRIGHTWIDTH);
+               break;
+            case PtBorderBottomWidth:
+               TtaWriteByte (outfile, C_PR_BORDERBOTTOMWIDTH);
+               break;
+            case PtBorderLeftWidth:
+               TtaWriteByte (outfile, C_PR_BORDERLEFTWIDTH);
+               break;
+            case PtBorderTopColor:
+               TtaWriteByte (outfile, C_PR_BORDERTOPCOLOR);
+               break;
+            case PtBorderRightColor:
+               TtaWriteByte (outfile, C_PR_BORDERRIGHTCOLOR);
+               break;
+            case PtBorderBottomColor:
+               TtaWriteByte (outfile, C_PR_BORDERBOTTOMCOLOR);
+               break;
+            case PtBorderLeftColor:
+               TtaWriteByte (outfile, C_PR_BORDERLEFTCOLOR);
+               break;
+            case PtBorderTopStyle:
+               TtaWriteByte (outfile, C_PR_BORDERTOPSTYLE);
+               break;
+            case PtBorderRightStyle:
+               TtaWriteByte (outfile, C_PR_BORDERRIGHTSTYLE);
+               break;
+            case PtBorderBottomStyle:
+               TtaWriteByte (outfile, C_PR_BORDERBOTTOMSTYLE);
+               break;
+            case PtBorderLeftStyle:
+               TtaWriteByte (outfile, C_PR_BORDERLEFTSTYLE);
+               break;
 	    case PtSize:
 	       TtaWriteByte (outfile, C_PR_SIZE);
 	       break;
@@ -1056,12 +1120,16 @@ PtrPRule            pPRule;
 		 case PresImmediate:
 		    switch (currentRule->PrType)
 			  {
+			     case PtFunction:
 			     case PtVisibility:
 			     case PtDepth:
-			     case PtFunction:
 			     case PtFillPattern:
 			     case PtBackground:
 			     case PtForeground:
+                             case PtBorderTopColor:
+                             case PtBorderRightColor:
+                             case PtBorderBottomColor:
+			     case PtBorderLeftColor:
 				WriteBoolean (currentRule->PrAttrValue);
 				WriteSignedShort (currentRule->PrIntValue);
 				break;
@@ -1071,6 +1139,10 @@ PtrPRule            pPRule;
 			     case PtUnderline:
 			     case PtThickness:
 			     case PtLineStyle:
+			     case PtBorderTopStyle:
+			     case PtBorderRightStyle:
+			     case PtBorderBottomStyle:
+			     case PtBorderLeftStyle:
 				TtaWriteByte (outfile, currentRule->PrChrValue);
 				break;
 			     case PtBreak1:
@@ -1079,6 +1151,18 @@ PtrPRule            pPRule;
 			     case PtSize:
 			     case PtLineSpacing:
 			     case PtLineWeight:
+			     case PtMarginTop:
+			     case PtMarginRight:
+			     case PtMarginBottom:
+			     case PtMarginLeft:
+			     case PtPaddingTop:
+			     case PtPaddingRight:
+			     case PtPaddingBottom:
+			     case PtPaddingLeft:
+			     case PtBorderTopWidth:
+			     case PtBorderRightWidth:
+			     case PtBorderBottomWidth:
+			     case PtBorderLeftWidth:
 				WriteUnit (currentRule->PrMinUnit);
 				WriteBoolean (currentRule->PrMinAttr);
 				WriteSignedShort (currentRule->PrMinValue);
@@ -1250,7 +1334,7 @@ PtrSSchema          pSS;
 	TtaWriteByte (outfile, pConst->PdAlphabet);
 	j = 0;
 	do
-	   TtaWriteByte (outfile, pConst->PdString[j++]);
+	   TtaWriteWideChar (outfile, pConst->PdString[j++], ISO_8859_1 /* UTF_8 */);
 	while (pConst->PdString[j - 1] != '\0');
      }
 
