@@ -181,7 +181,7 @@ PtrElement	    *pSecondPart;
 	     if (CallEventType ((NotifyEvent *) & notifyEl, TRUE))
 		/* l'application refuse */
 		dontCut = TRUE;
-	     else if (rank <= pEl->ElTextLength && rank > 1)
+	     else if (rank <= pEl->ElTextLength)
 		/* on va couper le texte */
 	       {
 		  /* envoie l'evenement ElemTextModify.Pre a qui le demande */
@@ -201,19 +201,13 @@ PtrElement	    *pSecondPart;
 	/* copie l'element de texte avec attributs et presentation specifique */
 	pEl2 = ReplicateElement (pEl, pDoc);
 	pEl2->ElVolume = 0;
+	pEl2->ElTextLength = 0;
 	*pSecondPart = pEl2;
 	/* acquiert un buffer de texte pour la 2eme partie */
 	CreateTextBuffer (pEl2);
 	/* chaine le nouvel element */
-	if (rank == 1)
-	  {
-	   InsertElementBefore (pEl, pEl2);
-	   pEl2->ElTextLength = 0;
-	   pEl2->ElVolume = 0;
-	  }
-	else
-	   InsertElementAfter (pEl, pEl2);
-	if (rank <= pEl->ElTextLength && rank > 1)
+	InsertElementAfter (pEl, pEl2);
+	if (rank <= pEl->ElTextLength)
 	   /* le nouvel element n'a pas une longueur nulle */
 	  {
 	     /* longueur du nouvel element */
@@ -784,11 +778,11 @@ int                *len;
   ----------------------------------------------------------------------*/
 
 #ifdef __STDC__
-void                CopyStringToText (STRING srceStrn, PtrTextBuffer pCopyBuf, int *LgCopiee)
+void                CopyStringToText (CHAR_T* srceStrn, PtrTextBuffer pCopyBuf, int *LgCopiee)
 
 #else  /* __STDC__ */
 void                CopyStringToText (srceStrn, pCopyBuf, LgCopiee)
-STRING              srceStrn;
+CHAR_T*             srceStrn;
 PtrTextBuffer       pCopyBuf;
 int                *LgCopiee;
 
@@ -796,7 +790,7 @@ int                *LgCopiee;
 
 {
    PtrTextBuffer       pTBDest;
-   STRING              pSrce;
+   CHAR_T*             pSrce;
    int                 lenRest, len;
 
    *LgCopiee = 0;
@@ -833,7 +827,7 @@ int                *LgCopiee;
 		  ustrncpy (pTBDest->BuContent + pTBDest->BuLength, pSrce, len);
 		  pSrce += len;
 		  pTBDest->BuLength += len;
-		  pTBDest->BuContent[pTBDest->BuLength] = EOS;
+		  pTBDest->BuContent[pTBDest->BuLength] = WC_EOS;
 		  lenRest -= len;
 	       }
 	  }
@@ -848,12 +842,12 @@ int                *LgCopiee;
   ----------------------------------------------------------------------*/
 
 #ifdef __STDC__
-void                CopyTextToString (PtrTextBuffer pSrceBuf, STRING pStrCpy, int *len)
+void                CopyTextToString (PtrTextBuffer pSrceBuf, CHAR_T* pStrCpy, int *len)
 
 #else  /* __STDC__ */
 void                CopyTextToString (pSrceBuf, pStrCpy, len)
 PtrTextBuffer       pSrceBuf;
-STRING              pStrCpy;
+CHAR_T*             pStrCpy;
 int                *len;
 
 #endif /* __STDC__ */
