@@ -32,46 +32,47 @@
 
 extern struct Langue_Ctl LangTable[MAX_LANGUAGES];
 extern struct Langue_Ctl TypoLangTable[MAX_LANGUAGES];
-extern int          FreeEntry;
+extern int               FreeEntry;
+extern CHARSET           CharEncoding;
 
-static STRING       dictPath;	/* environment variable DICOPAR */
+static CHAR_T*      dictPath;	/* environment variable DICOPAR */
 static PtrDict      dictTable[MaxDictionaries];
 
 unsigned            ReverseCode[NbLtr];
-UCHAR_T       Code[256];	/* Alphabet characters */
+unsigned char       Code[256];	/* Alphabet characters */
 
 #include "memory_f.h"
 #include "fileaccess_f.h"
 #include "thotmsg_f.h"
 #include "platform_f.h"
-
+#include "ustring_f.h"
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
 static void           LoadAlphabet ()
 {
-  FILE               *falpha;
-  Buffer              alphaName;
-  UCHAR_T       x;
-  int                 i;
+  FILE*         falpha;
+  PathBuffer    alphaName;
+  unsigned char x;
+  int        i;
   
   if (dictPath != NULL)
     ustrcpy (alphaName, dictPath);
   else
-    ustrcpy (alphaName, _EMPTYSTR_);
+    ustrcpy (alphaName, TEXT(""));
 
-  ustrcat (alphaName, DIR_STR);
+  ustrcat (alphaName, WC_DIR_STR);
   ustrcat (alphaName, TEXT("alphabet"));
   
   if ((falpha = ufopen (alphaName, TEXT("r"))) != NULL)
     {
       for (i = 0; i < 256; i++)
-	Code[i] = (UCHAR_T) 100;
+          Code[i] = (unsigned char) 100;
       i = 1;
       while ((fscanf (falpha, "%c ", &x) != EOF) && (i < NbLtr))
 	{
-	  Code[x] = (UCHAR_T) i;
-	  ReverseCode[i++] = (UCHAR_T) x;
+	  Code[x] = (unsigned char) i;
+	  ReverseCode[i++] = (unsigned char) x;
 	}
       fclose (falpha);
     }
@@ -140,20 +141,23 @@ PtrDict            *pDictionary;
 #endif /* __STDC__ */
 {
    int                 d;
+   PtrDict             pdict;
 
    if (*pDictionary != NULL)
       /* looks in the table for the descriptor of the dictionary to release */
      {
+       pdict = *pDictionary;
 	d = 0;
-	while (d < MaxDictionaries && dictTable[d] != *pDictionary)
+	while (d < MaxDictionaries && dictTable[d] != pdict)
 	   d++;
-	if (dictTable[d] == *pDictionary)
+	if (dictTable[d] == pdict)
 	  {
+	     TtaFreeMemory (pdict->DictDirectory);
 	     /* Releases the string and the list of words */
-	     FreeStringInDict (*pDictionary);
-	     FreeDictionary (*pDictionary);
+	     FreeStringInDict (pdict);
+	     FreeDictionary (pdict);
 	     dictTable[d] = NULL;
-	     *pDictionary = NULL;
+	     pdict = NULL;
 	  }
      }
 }
@@ -217,7 +221,7 @@ PtrDocument         document;
    a pointer (pDictionary) referencing its descriptor or NULL    
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-static void         SearchDictName (PtrDict * pDictionary, STRING dictName, STRING dictDirectory)
+static void         SearchDictName (PtrDict * pDictionary, CHAR_T* dictName, CHAR_T* dictDirectory)
 #else  /* __STDC__ */
 static void         SearchDictName (pDictionary, dictName, dictDirectory)
 PtrDict            *pDictionary;
@@ -250,18 +254,18 @@ STRING              dictDirectory;
    returns  1 if the file .DCT exists (treated)                      
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-static int          TestDictionary (char* dictName, char* dictDirectory)
+static int          TestDictionary (CHAR_T* dictName, CHAR_T* dictDirectory)
 #else  /* __STDC__ */
 static int          TestDictionary (dictName, dictDirectory)
-char*               dictName;
-char*               dictDirectory;
+CHAR_T*             dictName;
+CHAR_T*             dictDirectory;
 
 #endif /* __STDC__ */
 {
   int                 ret, i;
-  char                tempbuffer[THOT_MAX_CHAR];
+  CHAR_T              tempbuffer[THOT_MAX_CHAR];
 
-  FindCompleteName (dictName, "dic", dictDirectory, tempbuffer, &i);
+  FindCompleteName (dictName, TEXT("dic"), dictDirectory, tempbuffer, &i);
   if (TtaFileExist (tempbuffer) == 0)	/* Unknown file */
     {
       /* Looks for not pre-treated dictionary */
@@ -269,7 +273,7 @@ char*               dictDirectory;
       if (TtaFileExist (tempbuffer) == 0)
 	{
 	  /* File .DCT unknown: looks for a dictionary LEX not pre-treated */
-	  FindCompleteName (dictName, _LEXCST_, dictDirectory, tempbuffer, &i);
+	  FindCompleteName (dictName, TEXT("LEX"), dictDirectory, tempbuffer, &i);
 	  if (TtaFileExist (tempbuffer) == 0)
 	    /* unknown file */
 	    ret = -1;
@@ -307,7 +311,11 @@ PtrDict             dict;
       i = 0;
       while (i < dict->DictNbChars)
 	{
+#     ifdef _I18N_
+      TtaReadWideChar (dictFile, &(dict->DictString[i]), CharEncoding);
+#     else  /* !_I18N_ */
 	  TtaReadByte (dictFile, &(dict->DictString[i]));
+#     endif /* !_I18N_ */
 	  i++;
 	}
       
@@ -385,8 +393,7 @@ PtrDict             dict;
 	  /* not the end of the dictionary */
 	  {
 	     /* impossible to load the dictionary */
-	     TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_ERR_LOADING_DICO),
-				dict->DictName);
+	     TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_ERR_LOADING_DICO), dict->DictName);
 	     /* Release the dictionary */
 	     ReleaseDictionary (&dict);	/* => dict = nil */
 	     return (0);
@@ -413,13 +420,13 @@ PtrDict             dict;
    retourne dans pDictionary le pointeur sur son descripteur ou NULL        
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-static void         PrepareDictionary (PtrDict * pDictionary, STRING dictName, PtrDocument document, STRING dictDirectory, Language lang, ThotBool readonly, ThotBool treated, ThotBool toTreat)
+static void         PrepareDictionary (PtrDict * pDictionary, CHAR_T* dictName, PtrDocument document, CHAR_T* dictDirectory, Language lang, ThotBool readonly, ThotBool treated, ThotBool toTreat)
 #else  /* __STDC__ */
 static void         PrepareDictionary (pDictionary, dictName, document, dictDirectory, lang, readonly, treated, toTreat)
 PtrDict            *pDictionary;
-STRING              dictName;
+CHAR_T*             dictName;
 PtrDocument         document;
-STRING              dictDirectory;
+CHAR_T*             dictDirectory;
 Language            lang;
 ThotBool            readonly;
 ThotBool            treated;
@@ -442,21 +449,21 @@ ThotBool            toTreat;
       if (toTreat)
 	FindCompleteName (dictName, TEXT("DCT"), dictDirectory, tempbuffer, &i);
       else
-	FindCompleteName (dictName, _LEXCST_, dictDirectory, tempbuffer, &i);
+	FindCompleteName (dictName, TEXT("LEX"), dictDirectory, tempbuffer, &i);
     }
   if (readonly == FALSE)
     {
       /* Alterable dictionary */
       if (TtaFileExist (tempbuffer) != 0)
 	{
-	  dictFile = ufopen (tempbuffer, _RW_MODE_);
+	  dictFile = ufopen (tempbuffer, TEXT("rw"));
 	  /* updating the dictionary */
 	  TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_DICO), dictName);
 	}
       else
 	{
 	  new = TRUE;
-	  dictFile = ufopen (tempbuffer, _AppendMODE_);
+	  dictFile = ufopen (tempbuffer, TEXT("w+"));
 	  /* new dictionary */
 	  TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_NEW_DICO), dictName);
 	}
@@ -489,7 +496,7 @@ ThotBool            toTreat;
   pdict = *pDictionary;
   pdict->DictDoc = document;
   pdict->DictLanguage = lang;
-  ustrcpy (pdict->DictDirectory, dictDirectory);
+  pdict->DictDirectory = ustrdup (dictDirectory);
   ustrcpy (pdict->DictName, dictName);
   pdict->DictReadOnly = readonly;
   
@@ -588,14 +595,14 @@ ThotBool            toTreat;
   toCreate = TRUE
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-int                 LoadTreatedDict (PtrDict * pDictionary, Language lang, PtrDocument document, STRING dictName, STRING dictDirectory, ThotBool readonly, ThotBool toCreate)
+int                 LoadTreatedDict (PtrDict * pDictionary, Language lang, PtrDocument document, CHAR_T* dictName, CHAR_T* dictDirectory, ThotBool readonly, ThotBool toCreate)
 #else  /* __STDC__ */
 int                 LoadTreatedDict (pDictionary, lang, document, dictName, dictDirectory, readonly, toCreate)
 PtrDict            *pDictionary;
 Language            lang;
 PtrDocument         document;
-STRING              dictName;
-STRING              dictDirectory;
+CHAR_T*             dictName;
+CHAR_T*             dictDirectory;
 ThotBool            readonly;
 ThotBool            toCreate;
 #endif /* __STDC__ */
@@ -674,7 +681,8 @@ PtrDict            *pDictionary;
    PtrDict             pdict;
    PtrDocument         document;
    int                 d;
-   Name                dictName;
+   CHAR_T              dictName[MAX_NAME_LENGTH];
+   CHAR_T              *dictDirectory;
 
    document = NULL;
    if (*pDictionary == NULL)
@@ -692,6 +700,8 @@ PtrDict            *pDictionary;
 	     /* Getting information about the dictionary */
 	     ustrcpy (dictName, pdict->DictName);
 	     document = pdict->DictDoc;
+	     /* save the dictionary directory */
+             dictDirectory = pdict->DictDirectory;
 	     /* Release the string and the list of words ... */
 	     FreeStringInDict (pdict);
 	     FreeDictionary (pdict);
@@ -700,8 +710,9 @@ PtrDict            *pDictionary;
 	  }
      }
 
-   d = LoadTreatedDict (pDictionary, EOS, document, dictName,
-		 document->DocDirectory, FALSE, TRUE);
+   d = LoadTreatedDict (pDictionary, 0, document, dictName, dictDirectory, FALSE, TRUE);
+   TtaFreeMemory (dictDirectory);
+
    if (d == -1)
       return (FALSE);
 
@@ -764,8 +775,9 @@ Language            languageId;
    if (LangTable[lang].LangDict[0] == NULL)
      {
 	/* Loading the main dictionary */
-	if (LangTable[lang].LangPrincipal[0] != EOS)
+	if (LangTable[lang].LangPrincipal[0] != EOS && !LangTable[lang].LangDict[0])
 	  {
+         dictPtr = NULL;
 	     LoadTreatedDict (&dictPtr, lang, NULL, LangTable[lang].LangPrincipal, dictPath, TRUE, FALSE);
 	     if (dictPtr != NULL)
 		LangTable[lang].LangDict[0] = (Dictionary) dictPtr;
@@ -776,8 +788,9 @@ Language            languageId;
    if (LangTable[lang].LangDict[1] == NULL)
      {
 	/* Loading the secondary dictionary */
-	if (LangTable[lang].LangSecondary[0] != EOS)
+	if (LangTable[lang].LangSecondary[0] != EOS && !LangTable[lang].LangDict[1])
 	  {
+         dictPtr = NULL;
 	     LoadTreatedDict (&dictPtr, lang, NULL, LangTable[lang].LangSecondary, dictPath, TRUE, FALSE);
 	     if (dictPtr != NULL)
 		LangTable[lang].LangDict[1] = (Dictionary) dictPtr;
