@@ -112,18 +112,17 @@ STRING              cssToPrint;
 Document            document;
 #endif /* __STDC__ */
 { 
-   char*                   ptr;
-#ifdef _WINDOWS
+   CHAR_T*                 ptr;
+#  ifdef _WINDOWS
    static LPPRINTER_INFO_5 pInfo5;
-   STRING                  printArgv [100];
-   DWORD                   dwNeeded, dwReturned;
-   HANDLE                  hLib;
+   CHAR_T*                 printArgv [100];
+   HINSTANCE               hLib;
    FARPROC                 ptrMainProc;
    int                     printArgc = 0;
-#else  /* !_WINDOWS */
-   CHAR_T                  cmd[1024];
+#  else  /* !_WINDOWS */
+   char                    cmd[1024];
    int                     res;
-#endif /* _WINDOWS */
+#  endif /* _WINDOWS */
    int                     i, j = 0;
    int                     frame;
 
@@ -132,7 +131,7 @@ Document            document;
 #ifdef _WINDOWS
    printArgv[printArgc] = TtaAllocString (ustrlen (BinariesDirectory) + 7);
    ustrcpy (printArgv[printArgc], BinariesDirectory);
-   ustrcat (printArgv[printArgc], DIR_STR);
+   ustrcat (printArgv[printArgc], WC_DIR_STR);
    ustrcat (printArgv[printArgc], TEXT("print"));
    printArgc++;   
    printArgv[printArgc] = TtaAllocString (6) ;
@@ -308,11 +307,13 @@ Document            document;
      }
 
    /* transmit page format */
-   if (ustrcmp (PageSize, _A4PaperFormat_))
+   if (ustrcmp (PageSize, TEXT("A4")))
      {
 #ifdef _WINDOWS
+       CHAR_T tmp [MAX_TXT_LEN];
+       usprintf (tmp, TEXT("-P%s"), PageSize);
        printArgv[printArgc] = TtaAllocString (ustrlen (PageSize) + 3);
-       usprintf (printArgv[printArgc], TEXT("-P%s"), PageSize);
+       ustrcpy (printArgv[printArgc], tmp);
        printArgc++;
 #else  /* _WINDOWS */
        ustrcat (cmd, " -P");
@@ -450,7 +451,7 @@ Document            document;
      {
        /* insert the flag -v before each view name */
 #ifdef _WINDOWS
-       printArgv[printArgc] = TtaStrdup (TEXT("-v"));
+       printArgv[printArgc] = TtaWCSdup (TEXT("-v"));
        printArgc++;
        printArgv[printArgc] = TtaAllocString (50);
        j = 0;
@@ -475,7 +476,7 @@ Document            document;
 #ifdef _WINDOWS
 		   printArgv[printArgc][j++] = EOS;
 		   printArgc++;
-		   printArgv[printArgc] = TtaStrdup (TEXT("-v"));
+		   printArgv[printArgc] = TtaWCSdup (TEXT("-v"));
 		   printArgc++;
 		   printArgv[printArgc] = TtaAllocString (50);
 		   j = 0;
@@ -517,7 +518,7 @@ Document            document;
 	 {
 	   /* insert the flag -v before each view name */
 #ifdef _WINDOWS
-	   printArgv[printArgc] = TtaStrdup (TEXT("-css"));
+	   printArgv[printArgc] = TtaWCSdup (TEXT("-css"));
 	   printArgc++;
 	   printArgv[printArgc] = TtaAllocString (50);
 	   j = 0;
@@ -541,7 +542,7 @@ Document            document;
 #ifdef _WINDOWS
 		       printArgv[printArgc][j++] = EOS;
 		       printArgc++;
-		       printArgv[printArgc] = TtaStrdup (TEXT("-css"));
+		       printArgv[printArgc] = TtaWCSdup (TEXT("-css"));
 		       printArgc++;
 		       printArgv[printArgc] = TtaAllocString (50);
 		       j = 0;
@@ -573,27 +574,12 @@ Document            document;
    }
    /* transmit the path or source file */
 #ifdef _WINDOWS 
-   printArgv[printArgc] = TtaStrdup (TEXT("-removedir"));
+   printArgv[printArgc] = TtaWCSdup (TEXT("-removedir"));
    printArgc++;
    printArgv[printArgc] = TtaAllocString (ustrlen (dir) + ustrlen (name) + 6);
    usprintf  (printArgv[printArgc], TEXT("%s\\%s.PIV"), dir, name);
    printArgc++;
    WIN_ReleaseDeviceContext ();
-   if (buttonCommand || TtPrinterDC == 0) {
-       EnumPrinters (PRINTER_ENUM_LOCAL, NULL, 5, (LPBYTE) "", 0, &dwNeeded, &dwReturned) ;
-
-       // Alloue de lespace pour le tableau PRINTER_INFO_5
-       if (pInfo5)
-          HeapFree (GetProcessHeap (), 0, pInfo5) ;
-
-       pInfo5 = (LPPRINTER_INFO_5) HeapAlloc (GetProcessHeap (), HEAP_NO_SERIALIZE, dwNeeded);
-
-       // Enfin, remplit le tableau PRINTER_INFO_5
-       if (!pInfo5 || !EnumPrinters (PRINTER_ENUM_LOCAL, NULL, 5, (LPBYTE) pInfo5, dwNeeded, &dwNeeded, &dwReturned))
-          MessageBox (FrRef[currentFrame], TEXT("No printer available !"), NULL, MB_ICONSTOP);      
-       else 
-           TtPrinterDC = CreateDC (NULL, pInfo5->pPrinterName,  NULL, NULL);
-   }
 
    hLib = LoadLibrary (TEXT("thotprinter"));
    if (!hLib)
@@ -604,8 +590,8 @@ Document            document;
       return /* FATAL_EXIT_CODE */;
    }
 
-   ptrMainProc (FrRef [currentFrame], printArgc, printArgv, TtPrinterDC, TtIsTrueColor, TtWDepth, name, dir, hInstance, buttonCommand);
-   /* PrintDoc (FrRef [currentFrame], printArgc, printArgv, TtPrinterDC, TtIsTrueColor, TtWDepth, name, dir, hInstance, buttonCommand); */
+   ptrMainProc (FrRef [currentFrame], printArgc, printArgv, TtPrinterDC, TtIsTrueColor, 
+	            TtWDepth, name, dir, hInstance, buttonCommand);
    FreeLibrary (hLib);
 
    if (!IsWindowEnabled (FrRef[currentFrame]))
@@ -640,8 +626,8 @@ Document document;
 #endif /* __STDC__ */
 {
    PtrDocument pDoc;
-   char*               ptr;
-   int                 lg;
+   CHAR_T*     ptr;
+   int         lg;
 
    if (document == 0)
      pDoc = 0;
@@ -655,10 +641,10 @@ Document document;
        /* read DEFAULTPRINTER variable */
        ptr = TtaGetEnvString ("THOTPRINT");
        if (ptr == NULL)
-	 ustrcpy (pPrinter,_EMPTYSTR_);
+          ustrcpy (pPrinter, TEXT(""));
        else
-	 ustrcpy (pPrinter, ptr);
-       PSdir[0] = EOS;
+           ustrcpy (pPrinter, ptr);
+       PSdir[0] = WC_EOS;
        PrintingDoc = 0;
        defPaperPrint = TRUE;
        defManualFeed = FALSE;
@@ -685,9 +671,9 @@ Document document;
        PagesPerSheet = defPagesPerSheet;
        Paginate = defPaginate;
        if (defPageSize == PP_A4)
-         ustrcpy(PageSize, _A4PaperFormat_);
+          ustrcpy(PageSize, TEXT("A4"));
        else
-         ustrcpy(PageSize, _USPaperFormat_);
+           ustrcpy(PageSize, TEXT("US"));
        if (pDoc != NULL)
 	 {
 	   if (pDoc->DocDirectory[0] == DIR_SEP)
@@ -696,7 +682,7 @@ Document document;
 	     {
 	       ptr = NULL;
 	       ptr = TtaGetEnvString ("APP_TMPDIR");
-	       if (ptr == NULL || *ptr == EOS || !TtaCheckDirectory (ptr))
+	       if (ptr == NULL || *ptr == WC_EOS || !TtaCheckDirectory (ptr))
 		 {
 		   ptr = NULL;
 		   ptr = TtaGetEnvString ("TMPDIR");
@@ -757,7 +743,7 @@ STRING          *printDirName;
 #endif /* __STDC__ */
 {
    ThotPid             pid = ThotPid_get ();
-   char*               dirString;
+   CHAR_T*             dirString;
    int                 lg;
 
    *printDocName = PrintDocName;
@@ -783,10 +769,10 @@ STRING          *printDirName;
      }
 
    usprintf (PrintDocName, TEXT("Thot%ld"), (long) pid + numOfJobs);
-   ustrcpy (&PrintDirName[lg], DIR_STR);
+   ustrcpy (&PrintDirName[lg], WC_DIR_STR);
    ustrcat (&PrintDirName[lg], PrintDocName);
    if (!TtaCheckDirectory (PrintDirName))
-     TtaMakeDirectory (PrintDirName);
+      TtaMakeDirectory (PrintDirName);
 }
 
 /*----------------------------------------------------------------------
@@ -807,6 +793,7 @@ STRING              cssNames;
    Name                docName;
    Name                savePres, newPres;
    STRING              tmpDirName, tmpDocName;
+   CHAR_T              value[MAX_NAME_LENGTH];
    int                 orientation;
    ThotBool	       docReadOnly;
    ThotBool            ok;
@@ -815,14 +802,14 @@ STRING              cssNames;
    /* prepares the execution of the print command */
    ustrcpy (savePres, pDoc->DocSSchema->SsDefaultPSchema);
    if (PresSchema[0] != EOS)
-     ustrcpy (newPres, PresSchema);
+      ustrcpy (newPres, PresSchema);
    else
      ConfigGetPSchemaForPageSize (pDoc->DocSSchema, PageSize, newPres);
      
    if (newPres[0] != EOS)
       ustrcpy (pDoc->DocSSchema->SsDefaultPSchema, newPres);
    if (ThotLocalActions[T_rextprint]!=NULL && 
-       ustrcmp(pDoc->DocSSchema->SsDefaultPSchema, savePres))
+      ustrcmp(pDoc->DocSSchema->SsDefaultPSchema, savePres))
      {
        TtaDisplayMessage(INFO, TtaGetMessage(LIB,TMSG_CHANGE_PSCH), newPres);
      }
@@ -853,7 +840,9 @@ STRING              cssNames;
      }
 
    /* searches the paper orientation for the presentation scheme */
-   ConfigGetPresentationOption(pDoc->DocSSchema, TEXT("orientation"), Orientation);
+   ConfigGetPresentationOption(pDoc->DocSSchema, TEXT("orientation"), value);
+   if (value[0] != WC_EOS)
+     ustrcmp (Orientation, value);
    if (!ustrcmp (Orientation, TEXT("Landscape")))
      orientation = 1;
    else
@@ -922,77 +911,83 @@ PrintParameter parameter;
 int value;
 #endif /*__STDC__ */
 {
-   if (ThotLocalActions[T_rprint] == NULL)
-     /* force the initialization of printing parameters */
-     InitPrintParameters (0);
+  if (ThotLocalActions[T_rprint] == NULL)
+    /* force the initialization of printing parameters */
+    InitPrintParameters (0);
 
-   switch (parameter)
-     {
-     case PP_FirstPage:
-       if (value <0 || value >999)
-          TtaError(ERR_invalid_parameter);
-       else
-	 FirstPage = value;
-       break;
-     case PP_LastPage:
-       if (value <0 || value >999)
-	 TtaError(ERR_invalid_parameter);
-       else
-	 LastPage = value;
-       break;
-     case PP_Scale:
-       if (value <0 || value >999)
-	 TtaError(ERR_invalid_parameter);
-       else
-	 Reduction = value;
-       break;
-     case PP_NumberCopies:
-       if (value <0 || value >999)
-	 TtaError(ERR_invalid_parameter);
-       else
-	 NbCopies = value;
-       break;
-     case PP_Paginate:
-       if (value == PP_ON)
-	 Paginate = TRUE;
-       else if (value == PP_OFF)
-	 Paginate = FALSE;
-       else
-	 TtaError(ERR_invalid_parameter);
-       break;
-     case PP_ManualFeed:
-       if (value == PP_ON)
-	 ManualFeed = TRUE;
-       else if (value == PP_OFF)
-	 ManualFeed = FALSE;
-       else
-	 TtaError(ERR_invalid_parameter);
-       break;
-     case PP_PagesPerSheet:
-       if (value != 1 || value != 2 || value != 4)
-	 TtaError(ERR_invalid_parameter);
-       else
-	 PagesPerSheet = value;
-       break;
-     case PP_PaperSize:
-       if (value == PP_A4)
-	 ustrcpy (PageSize, _A4PaperFormat_);
-       else if (value == PP_US)
-	 ustrcpy (PageSize, _USPaperFormat_);
-       else
-	 TtaError(ERR_invalid_parameter);
-       break;
-     case PP_Destination:
-       if (value == PP_PRINTER)
-	 PaperPrint = TRUE;
-       else if (value == PP_PS)
-	 PaperPrint = FALSE;
-       else
-	 TtaError(ERR_invalid_parameter);
-       break;
-      default:
-       TtaError(ERR_invalid_parameter);
-     }
+  switch (parameter)
+    {
+    case PP_Orientation:
+      if (value == PP_Landscape)
+	ustrcpy (Orientation, TEXT("Landscape"));
+      else
+	ustrcpy (Orientation, TEXT("Portrait"));
+      break;
+    case PP_FirstPage:
+      if (value  <0 || value > 999)
+	TtaError(ERR_invalid_parameter);
+      else
+	FirstPage = value;
+      break;
+    case PP_LastPage:
+      if (value < 0 || value > 999)
+	TtaError(ERR_invalid_parameter);
+      else
+	LastPage = value;
+      break;
+    case PP_Scale:
+      if (value < 0 || value > 999)
+	TtaError(ERR_invalid_parameter);
+      else
+	Reduction = value;
+      break;
+    case PP_NumberCopies:
+      if (value < 0 || value > 999)
+	TtaError(ERR_invalid_parameter);
+      else
+	NbCopies = value;
+      break;
+    case PP_Paginate:
+      if (value == PP_ON)
+	Paginate = TRUE;
+      else if (value == PP_OFF)
+	Paginate = FALSE;
+      else
+	TtaError(ERR_invalid_parameter);
+      break;
+    case PP_ManualFeed:
+      if (value == PP_ON)
+	ManualFeed = TRUE;
+      else if (value == PP_OFF)
+	ManualFeed = FALSE;
+      else
+	TtaError(ERR_invalid_parameter);
+      break;
+    case PP_PagesPerSheet:
+      if (value != 1 && value != 2 && value != 4)
+	TtaError(ERR_invalid_parameter);
+      else
+	PagesPerSheet = value;
+      break;
+    case PP_PaperSize:
+      if (value == PP_A4)
+	ustrcpy (PageSize, TEXT("A4"));
+      else if (value == PP_US)
+	ustrcpy (PageSize, TEXT("US"));
+      else
+	TtaError(ERR_invalid_parameter);
+      break;
+    case PP_Destination:
+      if (value == PP_PRINTER)
+	PaperPrint = TRUE;
+      else if (value == PP_PS)
+	PaperPrint = FALSE;
+      else
+	TtaError(ERR_invalid_parameter);
+      break;
+    default:
+      TtaError(ERR_invalid_parameter);
+    }
 }
 
 
@@ -1009,6 +1004,12 @@ int value;
 {
   switch (parameter)
     {
+    case PP_Orientation:
+      if (!ustrcmp (Orientation, TEXT("Landscape")))
+	return (PP_Landscape);
+      else
+	return (PP_Portrait);
+      break;
     case PP_FirstPage:
       return (FirstPage);
       break;
@@ -1031,22 +1032,22 @@ int value;
       if (ManualFeed)
 	return (PP_ON);
       else
-	return (PP_ON);
+	return (PP_OFF);
       break;
     case PP_PagesPerSheet:
       return (PagesPerSheet);
       break;
     case PP_PaperSize:
-      if (!ustrcmp (PageSize, _A4PaperFormat_))
+      if (!ustrcmp (PageSize, TEXT("A4")))
 	return (PP_A4);
       else
 	return (PP_US);
       break;
     case PP_Destination:
       if (PaperPrint)
-	return (PP_ON);
+	return (PP_PRINTER);
       else
-	return (PP_ON);
+	return (PP_PS);
       break;
     default:
       TtaError(ERR_invalid_parameter);
@@ -1090,10 +1091,10 @@ STRING command;
   TtaSetPrintSchema fixes the printing schema.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                TtaSetPrintSchema (STRING name)
+void                TtaSetPrintSchema (CHAR_T* name)
 #else  /* __STDC__ */
 void                TtaSetPrintSchema (name)
-STRING name;
+CHAR_T*             name;
 #endif /* __STDC__ */
 {
   if (ustrlen(name) >= MAX_NAME_LENGTH)
@@ -1187,10 +1188,10 @@ STRING              txt;
 	  switch (val)
 	    {
 	    case 0:
-	      ustrcpy (PageSize, _A4PaperFormat_);
+	      ustrcpy (PageSize, TEXT("A4"));
 	      break;
 	    case 1:
-	      ustrcpy (PageSize, _USPaperFormat_);
+	      ustrcpy (PageSize, TEXT("US"));
 	      break;
 	    }
 	  break;
@@ -1280,7 +1281,7 @@ View                view;
    usprintf (&bufMenu[i], TEXT("B%s"), TtaGetMessage (LIB, TMSG_US));
    TtaNewSubmenu (NumMenuPaperFormat, NumFormPrint, 0, TtaGetMessage (LIB, TMSG_PAPER_SIZE), 2, bufMenu, NULL, FALSE);
 #  ifndef _WINDOWS
-   if (!ustrcmp (PageSize, _USPaperFormat_))
+   if (!ustrcmp (PageSize, TEXT("US")))
       TtaSetMenuForm (NumMenuPaperFormat, 1);
    else
       TtaSetMenuForm (NumMenuPaperFormat, 0);
