@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996 - 2000
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -58,11 +58,13 @@
 #include "thotmsg_f.h"
 #include "translation_f.h"
 #include "tree_f.h"
+#include "ustring_f.h"
 #include "viewapi_f.h"
 #include "views_f.h"
 #include "writepivot_f.h"
 
-static Name         nameBuffer;
+static CHAR_T nameBuffer[MAX_NATURES_DOC];
+static CHAR_T ISObuffer[MAX_NATURES_DOC];
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
@@ -79,6 +81,34 @@ PtrDocument         pDoc;
       (*ThotLocalActions[T_createtable]) (pEl, pDoc);
 }
 
+/*----------------------------------------------------------------------
+ * TtaGetDocumentCharset
+ *----------------------------------------------------------------------*/
+#ifdef __STDC__
+CHARSET TtaGetDocumentCharset (PtrDocument pDoc)
+#else  /* __STDC__ */
+CHARSET TtaGetDocumentCharset (pDoc)
+PtrDocument pDoc;
+#endif /* __STDC__ */
+{
+    return pDoc->Charset;
+}
+
+
+/*----------------------------------------------------------------------
+ * TtaSetDocumentCharset
+ *----------------------------------------------------------------------*/
+#ifdef __STDC__
+void TtaSetDocumentCharset (PtrDocument pDoc, CHARSET charSet)
+#else  /* __STDC__ */
+void TtaSetDocumentCharset (pDoc, charSet)
+PtrDocument pDoc;
+CHARSET     charSet;
+#endif /* __STDC__ */
+{
+    pDoc->Charset = charSet;
+}
+
 /*----------------------------------------------------------------------
    TtaOpenDocument
 
@@ -125,7 +155,7 @@ int                 accessMode;
 	     pDoc->DocDName[MAX_NAME_LENGTH - 1] = EOS;
 	     /* suppresses the .PIV suffix if found */
 	     if (lg > 4)
-		if (ustrcmp (&(pDoc->DocDName[lg - 4]), ".PIV") == 0)
+		if (ustrcmp (&(pDoc->DocDName[lg - 4]), TEXT(".PIV")) == 0)
 		   pDoc->DocDName[lg - 4] = EOS;
 	     GetDocIdent (&pDoc->DocIdent, pDoc->DocDName);
 	     ustrncpy (pDoc->DocDirectory, DocumentPath, MAX_PATH);
@@ -238,7 +268,7 @@ STRING              TSchemaName;
 
 
 /*----------------------------------------------------------------------
-   UnloadTree frees the document tree of pDoc				
+   UnloadTree free the document tree of pDoc				
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
 void                UnloadTree (Document document)
@@ -252,22 +282,22 @@ Document            document;
   pDoc = LoadedDocument[document - 1];
    if (pDoc != NULL)
      {
-       /* enleve la selection de ce document */
+       /* remove the selection on the document */
        if (ThotLocalActions[T_resetsel])
 	 (*ThotLocalActions[T_resetsel]) (pDoc);
 #ifndef NODISPLAY
-       /* libere le contenu du buffer s'il s'agit d'une partie de ce docum. */
+       /* remove the contents of the cut buffer related to the document */
        if (DocOfSavedElements == pDoc && ThotLocalActions[T_freesavedel])
 	 (*ThotLocalActions[T_freesavedel]) ();
-#endif
-       /* libere tous les arbres abstraits */
+#endif  /* NODISPLAY */
+       /* free the document tree */
        DeleteAllTrees (pDoc);
      }
 }
 
 
 /*----------------------------------------------------------------------
-   UnloadDocument libere le document pDoc				
+   UnloadDocument free the document contexts of pDoc				
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
 void                UnloadDocument (PtrDocument * pDoc)
@@ -276,30 +306,30 @@ void                UnloadDocument (pDoc)
 PtrDocument        *pDoc;
 #endif /* __STDC__ */
 {
-	int                 d;
+  int                 d;
 
-   if (*pDoc != NULL)
-      /* cherche dans la table le descripteur de document a liberer */
-     {
-	d = 0;
-	while (LoadedDocument[d] != *pDoc && d < MAX_DOCUMENTS - 1)
-	   d++;
-	if (LoadedDocument[d] == *pDoc)
-	  {
-	    /* libere les schemas */
-	    FreeDocumentSchemas (*pDoc);
-	    FreeDocument (LoadedDocument[d]);
-	    LoadedDocument[d] = NULL;
-	    /* annuler le pointeur sur le doc a imprimer */
+  if (*pDoc != NULL)
+    /* look for the current document in the documents table */
+    {
+      d = 0;
+      while (LoadedDocument[d] != *pDoc && d < MAX_DOCUMENTS - 1)
+	d++;
+      if (LoadedDocument[d] == *pDoc)
+	{
+	  /* free document schemas */
+	  FreeDocumentSchemas (*pDoc);
+	  FreeDocument (LoadedDocument[d]);
+	  LoadedDocument[d] = NULL;
 #ifndef NODISPLAY
-	    if (PrintingDoc == d+1)
-               PrintingDoc = 0;
-#endif
-	    *pDoc = NULL;
-	    /* libere les contextes inutilises */
-	    FreeAll ();
-	  }
-     }
+	  /* clear the PrintingDoc if it points to this document */
+	  if (PrintingDoc == d+1)
+	    PrintingDoc = 0;
+#endif /* NODISPLAY */
+	  *pDoc = NULL;
+	  /* clean up freed contexts */
+	  FreeAll ();
+	}
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -353,9 +383,9 @@ Document            document;
 	    CloseDocumentView (pDoc, numassoc, TRUE, FALSE);
 	  }
       UnloadTree (document);
-#else
+#else /* NODISPLAY */
       DeleteAllTrees (pDoc);
-#endif
+#endif /* NODISPLAY */
       UnloadDocument (&pDoc);
     }
 }
@@ -451,26 +481,26 @@ char*               path;
 
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-ThotBool            TtaIsInDocumentPath (PathBuffer directory)
+ThotBool            TtaIsInDocumentPath (CHAR_T* directory)
 
 #else  /* __STDC__ */
 ThotBool            TtaIsInDocumentPath (directory)
-char*               directory;
+CHAR_T*             directory;
 
 #endif /* __STDC__ */
 
 {
    int                 i;
-   STRING              ptr;
+   CHAR_T*             ptr;
 
    /* Verify if this directory is already in the list  */
    ptr = ustrstr (DocumentPath, directory);
    i = ustrlen (directory);
-   while (ptr != NULL && ptr[i] != PATH_SEP && ptr[i] != EOS)
+   while (ptr != NULL && ptr[i] != WC_PATH_SEP && ptr[i] != WC_EOS)
      {
-	ptr = ustrstr (ptr, PATH_STR);
-	if (ptr != NULL)
-	   ptr = ustrstr (ptr, directory);
+        ptr = ustrstr (ptr, WC_PATH_STR);
+        if (ptr != NULL)
+           ptr = ustrstr (ptr, directory);
      }
    return (ptr != NULL);
 }
@@ -488,11 +518,11 @@ char*               directory;
   ----------------------------------------------------------------------*/
 
 #ifdef __STDC__
-void                TtaAppendDocumentPath (STRING directory)
+void                TtaAppendDocumentPath (CHAR_T* directory)
 
 #else  /* __STDC__ */
 void                TtaAppendDocumentPath (directory)
-STRING              directory;
+CHAR_T*             directory;
 
 #endif /* __STDC__ */
 
@@ -516,8 +546,8 @@ STRING              directory;
 	else
 	  {
 	     if (i > 0)
-		ustrcat (DocumentPath, PATH_STR);
-	     ustrcat (DocumentPath, directory);
+            ustrcat (DocumentPath, WC_PATH_STR);
+         ustrcat (DocumentPath, directory);
 	  }
      }
 }
@@ -573,13 +603,13 @@ STRING              path;
   ----------------------------------------------------------------------*/
 
 #ifdef __STDC__
-SSchema             TtaNewNature (SSchema schema, STRING natureName, STRING presentationName)
+SSchema             TtaNewNature (SSchema schema, CHAR_T* natureName, CHAR_T* presentationName)
 
 #else  /* __STDC__ */
 SSchema             TtaNewNature (schema, natureName, presentationName)
 SSchema             schema;
-STRING              natureName;
-STRING              presentationName;
+CHAR_T*             natureName;
+CHAR_T*             presentationName;
 
 #endif /* __STDC__ */
 
@@ -595,8 +625,7 @@ STRING              presentationName;
      }
    else
      {
-	natureRule = CreateNature (natureName, presentationName,
-				   (PtrSSchema) schema);
+	natureRule = CreateNature (natureName, presentationName, (PtrSSchema) schema);
 	if (natureRule == 0)
 	  {
 	     TtaError (ERR_invalid_parameter);
@@ -627,13 +656,13 @@ STRING              presentationName;
   ----------------------------------------------------------------------*/
 
 #ifdef __STDC__
-SSchema             TtaNewSchemaExtension (Document document, STRING extensionName, STRING presentationName)
+SSchema             TtaNewSchemaExtension (Document document, CHAR_T* extensionName, CHAR_T* presentationName)
 
 #else  /* __STDC__ */
 SSchema             TtaNewSchemaExtension (document, extensionName, presentationName)
 Document            document;
-STRING              extensionName;
-STRING              presentationName;
+CHAR_T*             extensionName;
+CHAR_T*             presentationName;
 
 #endif /* __STDC__ */
 
@@ -654,8 +683,7 @@ STRING              presentationName;
    else
       /* parameter document is correct */
      {
-	extension = LoadExtension (extensionName, presentationName,
-				   LoadedDocument[document - 1]);
+	extension = LoadExtension (extensionName, presentationName, LoadedDocument[document - 1]);
 	if (extension == NULL)
 	  {
 	     TtaError (ERR_cannot_read_struct_schema);
@@ -678,11 +706,11 @@ STRING              presentationName;
 
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                TtaSetPSchema (Document document, STRING presentationName)
+void                TtaSetPSchema (Document document, CHAR_T* presentationName)
 #else  /* __STDC__ */
 void                TtaSetPSchema (document, presentationName)
 Document            document;
-STRING              presentationName;
+CHAR_T*             presentationName;
 #endif /* __STDC__ */
 
 {
@@ -697,53 +725,51 @@ STRING              presentationName;
    UserErrorCode = 0;
    /* verifies the parameter document */
    if (document < 1 || document > MAX_DOCUMENTS)
-	TtaError (ERR_invalid_document_parameter);
+      TtaError (ERR_invalid_document_parameter);
    else if (LoadedDocument[document - 1] == NULL)
-	TtaError (ERR_invalid_document_parameter);
+        TtaError (ERR_invalid_document_parameter);
    else
-      /* parameter document is correct */
-     {
-	pDoc = LoadedDocument[document - 1];
+       /* parameter document is correct */
+       {
+           pDoc = LoadedDocument[document - 1];
 #ifdef NODISPLAY
-	if (pDoc->DocSSchema != NULL)
-	   ustrncpy (pDoc->DocSSchema->SsDefaultPSchema, presentationName,
-		    MAX_NAME_LENGTH - 1);
+           if (pDoc->DocSSchema != NULL)
+               ustrncpy (pDoc->DocSSchema->SsDefaultPSchema, presentationName, MAX_NAME_LENGTH - 1);
 #else
-	/* verifies that there is no opened views */
-	ok = TRUE;
-	for (view = 1; view <= MAX_VIEW_DOC && ok; view++)
-	   if (pDoc->DocView[view - 1].DvPSchemaView != 0)
-	      ok = FALSE;
-	if (ok)
-	   for (Assoc = 1; Assoc <= MAX_ASSOC_DOC && ok; Assoc++)
-	      if (pDoc->DocAssocFrame[Assoc - 1] != 0)
-		 ok = FALSE;
-	if (!ok)
-	     TtaError (ERR_there_are_open_views);
-	else
-	   /* There is no opened views */
-	  {
-	     Name pschemaName;
+           /* verifies that there is no opened views */
+           ok = TRUE;
+           for (view = 1; view <= MAX_VIEW_DOC && ok; view++)
+               if (pDoc->DocView[view - 1].DvPSchemaView != 0)
+                  ok = FALSE;
+           if (ok)
+              for (Assoc = 1; Assoc <= MAX_ASSOC_DOC && ok; Assoc++)
+                  if (pDoc->DocAssocFrame[Assoc - 1] != 0)
+                     ok = FALSE;
+           if (!ok)
+              TtaError (ERR_there_are_open_views);
+           else
+               /* There is no opened views */
+               {
+                   Name pschemaName;
 
-	     ustrncpy(pschemaName, presentationName, MAX_NAME_LENGTH);
-	     if (pDoc->DocSSchema->SsPSchema != NULL)
-		/* a presentation schema already exist. One release it */
-	       {
-		  FreePresentationSchema (pDoc->DocSSchema->SsPSchema, pDoc->DocSSchema);
-		  pDoc->DocSSchema->SsPSchema = NULL;
-	       }
-	     /* Load the presentation schema */
-	     if (pDoc->DocSSchema->SsExtension)
-		/* to avoid that ReadPresentationSchema reloades the structure schema */
-		pDoc->DocSSchema->SsRootElem = 1;
-	     pDoc->DocSSchema->SsPSchema = LoadPresentationSchema (pschemaName,
-							  pDoc->DocSSchema);
-	     if (pDoc->DocSSchema->SsPSchema == NULL)
-		/* Failure while loading schema */
-		  TtaError (ERR_cannot_load_pschema);
-	  }
+                   ustrncpy (pschemaName, presentationName, MAX_NAME_LENGTH);
+                   if (pDoc->DocSSchema->SsPSchema != NULL)
+                      /* a presentation schema already exist. One release it */
+                      {
+                         FreePresentationSchema (pDoc->DocSSchema->SsPSchema, pDoc->DocSSchema);
+                         pDoc->DocSSchema->SsPSchema = NULL;
+                      }
+                   /* Load the presentation schema */
+                   if (pDoc->DocSSchema->SsExtension)
+                      /* to avoid that ReadPresentationSchema reloades the structure schema */
+                      pDoc->DocSSchema->SsRootElem = 1;
+                   pDoc->DocSSchema->SsPSchema = LoadPresentationSchema (pschemaName, pDoc->DocSSchema);
+                   if (pDoc->DocSSchema->SsPSchema == NULL)
+                      /* Failure while loading schema */
+                      TtaError (ERR_cannot_load_pschema);
+               }
 #endif
-     }
+      }
 }
 
 
@@ -758,11 +784,11 @@ STRING              presentationName;
 
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                TtaSetDocumentDirectory (Document document, STRING directory)
+void                TtaSetDocumentDirectory (Document document, CHAR_T* directory)
 #else  /* __STDC__ */
 void                TtaSetDocumentDirectory (document, directory)
 Document            document;
-STRING              directory;
+CHAR_T*             directory;
 #endif /* __STDC__ */
 
 {
@@ -828,7 +854,14 @@ int                 accessMode;
 		 pDoc->DocRootElement->ElAccess = AccessReadWrite;
 	     }
 #ifndef NODISPLAY
-	   SetAccessMode (LoadedDocument[document - 1], accessMode);
+	   /* update the paste entry */
+	   if (accessMode == 0)
+	     /* disable the Paste command */
+	     SwitchPaste (pDoc, FALSE);
+	   else if (FirstSavedElement != NULL || ClipboardThot.BuLength != 0)
+	     /* enable the Paste command */
+	     SwitchPaste (pDoc, TRUE);
+	   SetAccessMode (pDoc, accessMode);
 #endif
 	 }
      }
@@ -919,8 +952,11 @@ int                 notificationMode;
 }
 
 /*----------------------------------------------------------------------
-   SetDocumentModified set the document flag DocModified to the status.
+   SetDocumentModified set the document flags DocUpdated DocModified to
+   the status.
    The parameter length gives the number of characters added.
+   If the previous status of DocUpdated was FALSE the function notifies
+   the application.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
 void           SetDocumentModified (PtrDocument pDoc, ThotBool status, int length)
@@ -935,16 +971,18 @@ int            length;
     {
       if (status)
 	{
-	  if (!pDoc->DocModified && ThotLocalActions[T_docmodified])
+	  if (!pDoc->DocUpdated && ThotLocalActions[T_docmodified])
 	    (* ThotLocalActions[T_docmodified]) (IdentDocument (pDoc), TRUE);
 	  pDoc->DocModified = TRUE;
+	  pDoc->DocUpdated = TRUE;
 	  pDoc->DocNTypedChars += length;
 	}
       else
 	{
-	  if (pDoc->DocModified && ThotLocalActions[T_docmodified])
+	  if (pDoc->DocUpdated && ThotLocalActions[T_docmodified])
 	    (* ThotLocalActions[T_docmodified]) (IdentDocument (pDoc), FALSE);
 	  pDoc->DocModified = FALSE;
+	  pDoc->DocUpdated = FALSE;
 	  pDoc->DocNTypedChars = 0;
 	}
     }
@@ -967,7 +1005,6 @@ void                TtaSetDocumentModified (Document document)
 void                TtaSetDocumentModified (document)
 Document            document;
 #endif /* __STDC__ */
-
 {
    UserErrorCode = 0;
    /* verifies the parameter document */
@@ -998,7 +1035,6 @@ void                TtaSetDocumentUnmodified (Document document)
 void                TtaSetDocumentUnmodified (document)
 Document            document;
 #endif /* __STDC__ */
-
 {
    UserErrorCode = 0;
    /* verifies the parameter document */
@@ -1011,6 +1047,36 @@ Document            document;
      SetDocumentModified (LoadedDocument[document - 1], FALSE, 0);
 }
 
+/*----------------------------------------------------------------------
+   TtaSetDocumentUnupdated
+
+   Notifies the tool kit that a document must be considered as not updated
+   by the application or by the user. That will allow the application to
+   detect if any change will be made on the document
+   (see TtaIsDocumentUpdated).
+
+   Parameter:
+   document: the document.
+
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+void                TtaSetDocumentUnupdated (Document document)
+#else  /* __STDC__ */
+void                TtaSetDocumentUnupdated (document)
+Document            document;
+#endif /* __STDC__ */
+{
+   UserErrorCode = 0;
+   /* verifies the parameter document */
+   if (document < 1 || document > MAX_DOCUMENTS)
+     TtaError (ERR_invalid_document_parameter);
+   else if (LoadedDocument[document - 1] == NULL)
+     TtaError (ERR_invalid_document_parameter);
+   else
+     /* parameter document is correct */
+     LoadedDocument[document - 1]->DocUpdated = FALSE;
+}
+
 /*----------------------------------------------------------------------
    TtaGetDocumentName
 
@@ -1025,17 +1091,17 @@ Document            document;
   ----------------------------------------------------------------------*/
 
 #ifdef __STDC__
-STRING              TtaGetDocumentName (Document document)
+CHAR_T*             TtaGetDocumentName (Document document)
 
 #else  /* __STDC__ */
-STRING              TtaGetDocumentName (document)
+CHAR_T*             TtaGetDocumentName (document)
 Document            document;
 
 #endif /* __STDC__ */
 
 {
    UserErrorCode = 0;
-   nameBuffer[0] = EOS;
+   nameBuffer[0] = WC_EOS;
    /* verifies the parameter document */
    if (document < 1 || document > MAX_DOCUMENTS)
      {
@@ -1125,7 +1191,7 @@ int                 bufferLength;
 
 {
    UserErrorCode = 0;
-   nameBuffer[0] = EOS;
+   nameBuffer[0] = WC_EOS;
    /* verifies the parameter document */
    if (document < 1 || document > MAX_DOCUMENTS)
      {
@@ -1202,10 +1268,10 @@ Document            document;
   ----------------------------------------------------------------------*/
 
 #ifdef __STDC__
-STRING              TtaGetSSchemaName (SSchema schema)
+CHAR_T*             TtaGetSSchemaName (SSchema schema)
 
 #else  /* __STDC__ */
-STRING              TtaGetSSchemaName (schema)
+CHAR_T*             TtaGetSSchemaName (schema)
 SSchema             schema;
 
 #endif /* __STDC__ */
@@ -1214,14 +1280,14 @@ SSchema             schema;
    UserErrorCode = 0;
    if (schema == NULL)
      {
-	nameBuffer[0] = EOS;
+	ISObuffer[0] = WC_EOS;
 	TtaError (ERR_invalid_parameter);
      }
    else
      {
-	ustrcpy (nameBuffer, ((PtrSSchema) schema)->SsName);
+	ustrcpy (ISObuffer, ((PtrSSchema) schema)->SsName);
      }
-   return nameBuffer;
+   return ISObuffer;
 }
 
 /*----------------------------------------------------------------------
@@ -1239,10 +1305,10 @@ SSchema             schema;
   ----------------------------------------------------------------------*/
 
 #ifdef __STDC__
-STRING              TtaGetPSchemaName (SSchema schema)
+CHAR_T*              TtaGetPSchemaName (SSchema schema)
 
 #else  /* __STDC__ */
-STRING              TtaGetPSchemaName (schema)
+CHAR_T*              TtaGetPSchemaName (schema)
 SSchema             schema;
 
 #endif /* __STDC__ */
@@ -1251,49 +1317,51 @@ SSchema             schema;
    UserErrorCode = 0;
    if (schema == NULL)
      {
-	nameBuffer[0] = EOS;
+	ISObuffer[0] = EOS;
 	TtaError (ERR_invalid_parameter);
      }
    else
      {
-	ustrcpy (nameBuffer, ((PtrSSchema) schema)->SsDefaultPSchema);
+	ustrcpy (ISObuffer, ((PtrSSchema) schema)->SsDefaultPSchema);
      }
-   return nameBuffer;
+   return ISObuffer;
 }
 
 /* ChSchStruct recursively searches the schema which name is "name" within
    nature schema and extension schema used by pSS. It returns a pointer
    which references this schema or NULL if not found. */
 #ifdef __STDC__
-static SSchema      ChSchStruct (PtrSSchema pSS, STRING name)
+static SSchema      ChSchStruct (PtrSSchema pSS, CHAR_T* name)
 
 #else  /* __STDC__ */
 static SSchema      ChSchStruct (pSS, name)
 PtrSSchema          pSS;
-STRING              name;
+CHAR_T*             name;
 
 #endif /* __STDC__ */
 
 {
    int                 nRegle;
-   SSchema             retour;
+   SSchema             ret;
 
-   retour = NULL;
-   if (pSS != NULL)
+   ret = NULL;
+   if (pSS)
       if (ustrcmp (name, pSS->SsName) == 0)
 	 /* The schema itself */
-	 retour = (SSchema) pSS;
+	 ret = (SSchema) pSS;
       else
-	{
-	   /* Looks for the nature rule of the schema */
-	   for (nRegle = MAX_BASIC_TYPE - 1; retour == NULL && nRegle < pSS->SsNRules; nRegle++)
+	 {
+	 /* Looks for the nature rule of the schema */
+	 for (nRegle = pSS->SsNRules - 1;
+	      ret == NULL && nRegle >= MAX_BASIC_TYPE;
+	      nRegle--)
 	      if (pSS->SsRule[nRegle].SrConstruct == CsNatureSchema)
-		 retour = ChSchStruct (pSS->SsRule[nRegle].SrSSchemaNat, name);
-	   /* If not found, one search into the extension schema */
-	   if (retour == NULL)
-	      retour = ChSchStruct (pSS->SsNextExtens, name);
-	}
-   return retour;
+		 ret = ChSchStruct (pSS->SsRule[nRegle].SrSSchemaNat, name);
+	 /* If not found, one search into the extension schema */
+	 if (ret == NULL)
+	    ret = ChSchStruct (pSS->SsNextExtens, name);
+	 }
+   return ret;
 }
 
 /*----------------------------------------------------------------------
@@ -1312,10 +1380,10 @@ STRING              name;
 
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-SSchema             TtaGetSSchema (STRING name, Document document)
+SSchema             TtaGetSSchema (CHAR_T* name, Document document)
 #else  /* __STDC__ */
 SSchema             TtaGetSSchema (name, document)
-STRING              name;
+CHAR_T*             name;
 Document            document;
 #endif /* __STDC__ */
 
@@ -1324,7 +1392,7 @@ Document            document;
 
    UserErrorCode = 0;
    schema = NULL;
-   if (name == NULL || name[0] == EOS)
+   if (name == NULL || name[0] == WC_EOS)
      TtaError (ERR_invalid_parameter);
    /* verifies the parameter document */
    else if (document < 1 || document > MAX_DOCUMENTS)
@@ -1391,12 +1459,12 @@ SSchema             schema2;
 
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                TtaGiveSchemasOfDocument (STRING documentName, STRING structureName, STRING presentationName)
+void                TtaGiveSchemasOfDocument (STRING documentName, char* structureName, char* presentationName)
 #else  /* __STDC__ */
 void                TtaGiveSchemasOfDocument (documentName, structureName, presentationName)
 STRING              documentName;
-STRING              structureName;
-STRING              presentationName;
+char*               structureName;
+char*               presentationName;
 #endif /* __STDC__ */
 
 {
@@ -1405,7 +1473,7 @@ STRING              presentationName;
    CHAR_T              text[MAX_TXT_LEN];
    int                 i;
    ThotBool            error;
-   CHAR_T              charGotten;
+   char                charGotten;
    LabelString         lab;
    int                 currentVersion = 0;
 
@@ -1414,7 +1482,7 @@ STRING              presentationName;
    presentationName[0] = EOS;
    /* Arrange the name of the file to be opened with the documents directory name */
    ustrncpy (DirBuffer, DocumentPath, MAX_PATH);
-   MakeCompleteName (documentName, PIV_EXT2, DirBuffer, text, &i);
+   MakeCompleteName (documentName, TEXT("PIV"), DirBuffer, text, &i);
    /* Verify if the file exists */
    file = TtaReadOpen (text);
    if (file == 0)
@@ -1575,9 +1643,11 @@ SSchema            *nature;
 
 {
    PtrSSchema          nextNature;
+#ifndef NODISPLAY
    int                 n;
    PtrDocument         pDoc;
    ThotBool            found;
+#endif
 
    UserErrorCode = 0;
    /* verifies the parameter document */
@@ -1586,11 +1656,11 @@ SSchema            *nature;
 	TtaError (ERR_invalid_document_parameter);
    else if (LoadedDocument[document - 1] == NULL)
 	TtaError (ERR_invalid_document_parameter);
-   else
       /* parameter document is correct */
+#ifndef NODISPLAY
+   else
      {
 	pDoc = LoadedDocument[document - 1];
-#ifndef NODISPLAY
 	if (*nature == NULL)
 	   /* First apply, build the table of the natures of the document */
 	  {
@@ -1603,7 +1673,8 @@ SSchema            *nature;
 		n = 0;
 	  }
 	else
-	   /* The table is already builded. One looks for the current entry of the table */
+	   /* The table is already builded. One looks for the current entry
+	      of the table */
 	  {
 	     n = 1;
 	     found = FALSE;
@@ -1629,8 +1700,8 @@ SSchema            *nature;
 	     if (found)
 		nextNature = pDoc->DocNatureSSchema[n];
 	  }
-#endif
      }
+#endif
    *nature = (SSchema) nextNature;
 }
 
@@ -1672,6 +1743,44 @@ Document            document;
    return modified;
 }
 
+/*----------------------------------------------------------------------
+   TtaIsDocumentUpdated
+
+   Indicates whether a document has been modified by the user or not
+   since the last TtaSetDocumentUnupdated or TtaSetDocumentUnmodified.
+   Modifications made by the application program are not considered,
+   except when explicitely notified by TtaDocumentModified.
+
+   Parameter:
+   document: the document.
+
+   Return value:
+   1 if the document has been modified by the user since it has been saved,
+   loaded or created, 0 if it has not been modified.
+
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+int                 TtaIsDocumentUpdated (Document document)
+#else  /* __STDC__ */
+int                 TtaIsDocumentModified (document)
+Document            document;
+#endif /* __STDC__ */
+{
+   int                 updated;
+
+   UserErrorCode = 0;
+   /* verifies the parameter document */
+   updated = 0;
+   if (document < 1 || document > MAX_DOCUMENTS)
+     TtaError (ERR_invalid_document_parameter);
+   else if (LoadedDocument[document - 1] == NULL)
+     TtaError (ERR_invalid_document_parameter);
+   else if (LoadedDocument[document - 1]->DocUpdated)
+     /* parameter document is correct */
+      updated = 1;
+   return updated;
+}
+
 /*----------------------------------------------------------------------
    TtaGetDocumentAccessMode
 
