@@ -1267,7 +1267,6 @@ void                StructSelectLoadResources ()
      {
 	/* connecte les action d'edition structuree */
 	TteConnectAction (T_selecbox, (Proc) GetClickedStructBox);
-	TteConnectAction (T_switchsel, (Proc) SwitchSelection);
 	TteConnectAction (T_checksel, (Proc) CheckSelectedElement);
 	TteConnectAction (T_resetsel, (Proc) ResetSelection);
 	TteConnectAction (T_selstring, (Proc) SelectString);
@@ -1279,7 +1278,6 @@ void                StructSelectLoadResources ()
 	TteConnectAction (T_cmdpaste, (Proc) StructPasteCommand);
 	TteConnectAction (T_enter, (Proc) StructReturnKey);
 	MenuActionList[CMD_CreateElement].Call_Action = (Proc) StructReturnKey;
-	MenuActionList[CMD_CreateElement].User_Action = (UserProc) NULL;
 	InitSelection ();
      }
 }
