@@ -155,10 +155,11 @@ ThotBool            toend;
    xDelta and yDelta.                     
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-static void         LocateLeafBox (int frame, int x, int y, int xDelta, int yDelta, PtrBox pFrom, ThotBool extendSel)
+static void         LocateLeafBox (int frame, View view, int x, int y, int xDelta, int yDelta, PtrBox pFrom, ThotBool extendSel)
 #else  /* __STDC__ */
-static void         LocateLeafBox (frame, x, y, xDelta, yDelta, pFrom, extendSel)
+static void         LocateLeafBox (frame, view, x, y, xDelta, yDelta, pFrom, extendSel)
 int                 frame;
+View                view;
 int                 x;
 int                 y;
 int                 xDelta;
@@ -194,34 +195,50 @@ ThotBool            extendSel;
    doc = FrameTable[frame].FrDoc;
    GetSizesFrame (frame, &w, &h);
    if (xDelta < 0 && pFrame->FrXOrg > 0 && x + xDelta < 0)
-       TtcScrollLeft (doc, 1);
+       TtcScrollLeft (doc, view);
    else if (xDelta > 0 && x + xDelta > pFrame->FrXOrg + w)
-       TtcScrollRight (doc, 1);
-   else if (yDelta < 0 && pFrame->FrYOrg > 0 && y + yDelta < pFrame->FrYOrg)
+       TtcScrollRight (doc, view);
+   else if (yDelta < 0 && pFrame->FrAbstractBox->AbBox->BxNext &&
+	    y + yDelta < pFrame->FrYOrg + pFrame->FrAbstractBox->AbBox->BxNext->BxYOrg)
      {
+       /* try to select up to the top of the current displayed frame */
        do
 	 {
+	   /* scroll as long as the top of the view is not reached */
 	   org = pLastBox->BxYOrg;
-	   TtcLineUp (doc, 1);
+	   TtcLineUp (doc, view);
 	   /* update the new position */
 	   if (org != pLastBox->BxYOrg)
-	     y = y + org - pLastBox->BxYOrg;
+	     y = y - org + pLastBox->BxYOrg;
 	 }
-       while (pFrame->FrYOrg > 0 && y + yDelta < pFrame->FrYOrg &&
-	      (y > 0 || pFrame->FrAbstractBox->AbTruncatedHead));
+       while (y + yDelta < pFrame->FrYOrg &&
+	      /* we don't see the top of the box */
+	      (y < 0 ||
+	       /* the frame is not on the top of the concrete image */
+	       pFrame->FrYOrg > 0 ||
+	       /* we don't see the top of the document */
+	       pFrame->FrAbstractBox->AbTruncatedHead));
      }
-   else if (yDelta > 0 && y + yDelta > pFrame->FrYOrg + h)
+   else if (yDelta > 0 && pFrame->FrAbstractBox->AbBox->BxPrevious &&
+	    (y + yDelta > pFrame->FrYOrg + h ||
+	     y + yDelta > pFrame->FrAbstractBox->AbBox->BxPrevious->BxYOrg + pFrame->FrAbstractBox->AbBox->BxPrevious->BxHeight))
      {
+       /* try to select down to the bottom of the current displayed frame */
        do
 	 {
+	   /* scroll as long as the bootom of the view is not reached */
 	   org = pLastBox->BxYOrg;
-	   TtcLineDown (doc, 1);
+	   TtcLineDown (doc, view);
 	   /* update the new position */
 	   if (org != pLastBox->BxYOrg)
-	     y = y + org - pLastBox->BxYOrg;
+	     y = y - org + pLastBox->BxYOrg;
 	 }
        while (y + yDelta > pFrame->FrYOrg + h &&
-	      (y < pFrame->FrAbstractBox->AbBox->BxHeight || pFrame->FrAbstractBox->AbTruncatedTail));
+	      /* we don't see the bottom of the box */
+	      (/* the frame is not on the top of the concrete image */
+	       pFrame->FrYOrg + h < pFrame->FrAbstractBox->AbBox->BxHeight ||
+	       /* we don't see the top of the document */
+	       pFrame->FrAbstractBox->AbTruncatedTail));
      }
 
    pBox = GetLeafBox (pLastBox, frame, &x, &y, xDelta, yDelta);
@@ -439,7 +456,7 @@ ThotBool            extendSel;
 		   if (pBoxBegin->BxXOrg + pViewSel->VsXPos + 4 < pFrame->FrXOrg)
 		     {
 		       if (pFrame->FrXOrg > 0)
-			 TtcScrollLeft (doc, 1);
+			 TtcScrollLeft (doc, view);
 		     }
 		   else if (pBoxBegin->BxXOrg + pViewSel->VsXPos > pFrame->FrXOrg + w)
 		     HorizontalScroll (frame, pBoxBegin->BxXOrg + pViewSel->VsXPos - pFrame->FrXOrg - w, 0);
@@ -450,7 +467,7 @@ ThotBool            extendSel;
 		   x = pBox->BxXOrg + xpos;
 		   y = pBox->BxYOrg + (pBox->BxHeight / 2);
 		   xDelta = -2;
-		   LocateLeafBox (frame, x, y, xDelta, 0, pBox, extendSel);
+		   LocateLeafBox (frame, view, x, y, xDelta, 0, pBox, extendSel);
 		 }
 	     }
 	   /* Get the last X position */
@@ -504,7 +521,7 @@ ThotBool            extendSel;
 		   if (pBoxEnd->BxXOrg + pViewSelEnd->VsXPos > pFrame->FrXOrg + w)
 		     {
 		       if (FrameTable[frame].FrScrollOrg + FrameTable[frame].FrScrollWidth > pFrame->FrXOrg + w)
-			 TtcScrollRight (doc, 1);
+			 TtcScrollRight (doc, view);
 		     }
 		   else if (pBoxEnd->BxXOrg + pViewSelEnd->VsXPos - 4 < pFrame->FrXOrg)
 		     HorizontalScroll (frame, pBoxEnd->BxXOrg + pViewSelEnd->VsXPos - 4 - pFrame->FrXOrg, 0);
@@ -514,7 +531,7 @@ ThotBool            extendSel;
 		   x = pBox->BxXOrg + pBox->BxWidth;
 		   y = pBox->BxYOrg + (pBox->BxHeight / 2);
 		   xDelta = 2;
-		   LocateLeafBox (frame, x, y, xDelta, 0, pBox, extendSel);
+		   LocateLeafBox (frame, view, x, y, xDelta, 0, pBox, extendSel);
 		 }
 	     }
 	   /* Get the last X position */
@@ -573,7 +590,7 @@ ThotBool            extendSel;
 	       else
 		 RightExtended = TRUE;
 
-	       LocateLeafBox (frame, x, y, 0, yDelta, pBox, extendSel);
+	       LocateLeafBox (frame, view, x, y, 0, yDelta, pBox, extendSel);
 	     }
 	   break;
 	   
@@ -603,7 +620,7 @@ ThotBool            extendSel;
 	       else if (extendSel)
 		 LeftExtended = TRUE;
 
-	       LocateLeafBox (frame, x, y, 0, yDelta, pBox, extendSel);
+	       LocateLeafBox (frame, view, x, y, 0, yDelta, pBox, extendSel);
 	     }
 	   break;
 	 }
@@ -780,6 +797,7 @@ View                view;
       /* Rien a copier */
       return 0;
 
+   lg = 1;
    if (lastChar == 0)
       /* Il faut prendre tout le contenu de tout l'element */
       lastChar = pLastEl->ElVolume;
@@ -802,10 +820,13 @@ View                view;
 
 	     /* On ajoute le volume de l'element */
 	     if (pEl != NULL)
-		if (pEl == pLastEl)
+	       {
+		 if (pEl == pLastEl)
 		   maxLength += lastChar;
-		else
+		 else
 		   maxLength += pEl->ElVolume;
+		 lg += 2;
+	       }
 	  }
      }
 
@@ -814,7 +835,7 @@ View                view;
       return 0;
 
    /* On reserve un volant de 100 caracteres pour ajouter des CR */
-   max = maxLength + 100;
+   max = maxLength + lg;
    /* Alloue un Xbuffer de la longueur voulue */
    Xbuffer = TtaAllocString (max);
 
