@@ -23,6 +23,7 @@
 #include "pschema.h"
 #include "application.h"
 #include "style.h"
+#include "HTMLstyleColor.h"
 
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
@@ -34,6 +35,154 @@
 #include "memory_f.h"
 #include "style_f.h"
 
+/*----------------------------------------------------------------------
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+unsigned int TtaHexaVal (CHAR_T c)
+#else
+unsigned int TtaHexaVal (c)
+CHAR_T       c;
+#endif
+{
+   if (c >= TEXT('0') && c <= TEXT('9'))
+      return (c - TEXT('0'));
+   if (c >= TEXT('a') && c <= TEXT('f'))
+      return (c - TEXT('a') + 10);
+   if (c >= TEXT('A') && c <= TEXT('F'))
+      return (c - TEXT('A') + 10);
+   return (0);
+}
+
+/*----------------------------------------------------------------------
+  TtaGiveRGB
+  Returns the RGB of the color and the pointer to the following text in
+  value if the parsing was finished.
+ ----------------------------------------------------------------------*/
+#ifdef __STDC__
+CHAR_T            *TtaGiveRGB (CHAR_T *value, unsigned short *red, unsigned short *green, unsigned short *blue)
+#else  /* __STDC__ */
+  CHAR_T          *TtaGiveRGB (value, red, green, blue)
+CHAR_T            *value;
+unsigned short    *red;
+unsigned short    *green;
+unsigned short    *blue;
+#endif /* __STDC__ */
+{
+  CHAR_T              colname[100];
+  CHAR_T             *ptr;
+  int                 i, len;
+  int                 r, g, b;
+  ThotBool            failed;
+
+  ptr = value;
+  failed = TRUE;
+  *red = -1;
+  *green = 0;
+  *blue = 0;
+  if (*ptr == TEXT('#') ||
+      (isxdigit (ptr[0]) && isxdigit (ptr[1]) && isxdigit (ptr[2])))
+    {
+      if (*ptr == TEXT('#'))
+	ptr++;
+      /* we expect an hexa encoding like F00 or FF0000 */
+      if (isxdigit (ptr[0]) && isxdigit (ptr[1]) && isxdigit (ptr[2]))
+	{
+	  if (!isxdigit (ptr[3]))
+	    {
+	      /* encoded as on 3 digits #F0F  */
+	      *red = TtaHexaVal (ptr[0]) * 16 + TtaHexaVal (ptr[0]);
+	      *green = TtaHexaVal (ptr[1]) * 16 + TtaHexaVal (ptr[1]);
+	      *blue = TtaHexaVal (ptr[2]) * 16 + TtaHexaVal (ptr[2]);
+	      ptr = &ptr[3];
+	      failed = FALSE;
+	    }
+	  else if (isxdigit (ptr[4]) && isxdigit (ptr[5]))
+	    {
+	      /* encoded as on 3 digits #FF00FF */
+	      *red = TtaHexaVal (ptr[0]) * 16 + TtaHexaVal (ptr[1]);
+	      *green = TtaHexaVal (ptr[2]) * 16 + TtaHexaVal (ptr[3]);
+	      *blue = TtaHexaVal (ptr[4]) * 16 + TtaHexaVal (ptr[5]);
+	      ptr = &ptr[6];
+	      failed = FALSE;
+	    }
+	}
+    }
+  else if (!ustrncasecmp (ptr, TEXT("rgb"), 3))
+    {
+      ptr = &ptr[3];
+      ptr = TtaSkipWCBlanks (ptr);
+      if (*ptr == TEXT('('))
+	{
+	  ptr++;
+	  ptr = TtaSkipWCBlanks (ptr);
+	  failed = FALSE;
+	  if (*ptr == TEXT('%'))
+	    {
+	      /* encoded as rgb(%red,%green,&blue) */
+	      usscanf (ptr, TEXT("%%%d"), &r);
+	      while (*ptr != WC_EOS && *ptr != TEXT(','))
+		ptr++;
+	      ptr++;
+	      usscanf (ptr, TEXT("%%%d"), &g);
+	      while (*ptr != WC_EOS && *ptr != TEXT(','))
+		ptr++;
+	      ptr++;
+	      usscanf (ptr, TEXT("%%%d"), &b);
+	      *red = (unsigned short)(r * 255 / 100);
+	      *green = (unsigned short)(g * 255 / 100);
+	      *blue = (unsigned short)(b * 255 / 100);
+	    }
+	  else
+	    {
+	      /* encoded as rgb(red,green,blue) */
+	      usscanf (ptr, TEXT("%d"), &r);
+	      while (*ptr != WC_EOS && *ptr != TEXT(','))
+		ptr++;
+	      ptr++;
+	      usscanf (ptr, TEXT("%d"), &g);
+	      while (*ptr != WC_EOS && *ptr != TEXT(','))
+		ptr++;
+	      ptr++;
+	      usscanf (ptr, TEXT("%d"), &b);
+	      *red = (unsigned short)r;
+	      *green = (unsigned short)g;
+	      *blue = (unsigned short)b;
+	    }
+	  /* search the rgb end */
+	  while (*ptr != WC_EOS && *ptr != TEXT(')'))
+	    ptr++;
+	  ptr++;
+	}
+    }
+  else if (TtaIsAlpha (*ptr))
+    {
+      /* we expect a color name like "red", store it in colname */
+      len = (sizeof (colname) / sizeof (CHAR_T)) - 1;
+      for (i = 0; i < len && TtaIsAlnum (*ptr); i++)
+	{
+	  colname[i] = *ptr;
+	  ptr++;
+	}
+      colname[i] = WC_EOS;
+      
+      /* Lookup the color name in our own color name database */
+      for (i = 0; i < NBCOLORNAME && failed; i++)
+	if (!ustrcasecmp (ColornameTable[i].name, colname))
+	  {
+	    *red = ColornameTable[i].red;
+	    *green = ColornameTable[i].green;
+	    *blue = ColornameTable[i].blue;
+	    failed = FALSE;
+	  }
+      if (failed)
+	failed = ThotGiveRGB (colname, red, green, blue);
+    }
+  if (failed)
+    return (value);
+  else
+    return (ptr);
+}
+
 /*----------------------------------------------------------------------
   BuildBoxName : generate an unique name encoding for the given context.
   Assume the ancestor list has been sorted.
@@ -67,7 +216,7 @@ Name               *boxname;
     usprintf (&buffer[len], TEXT("%d.%s,"), ctxt->attrType[0], ctxt->attrText[0]);
   len = ustrlen (buffer);
 
-  ustrncpy ((STRING) boxname, buffer, sizeof (Name));
+  ustrncpy (*boxname, buffer, sizeof (Name));
 }
 
 /*----------------------------------------------------------------------
@@ -175,7 +324,8 @@ unsigned int     extra;
      {
        /* shortcut : rules are sorted by type and view number */
        if (cur->PrType > type ||
-	  (cur->PrType == type && type == PtFunction && cur->PrPresFunction > (FunctionType) extra))
+	  (cur->PrType == type && type == PtFunction &&
+	   cur->PrPresFunction > (FunctionType) extra))
 	 cur = NULL;
        else
 	 {
@@ -184,7 +334,8 @@ unsigned int     extra;
 	   if (cur->PrViewNum == 1 && cur->PrType == type &&
 	       (type != PRFunction ||
 	       /* check for extra specification in case of function rule */
-	       (type == PRFunction && cur->PrPresFunction == (FunctionType) extra)))
+	       (type == PRFunction &&
+		cur->PrPresFunction == (FunctionType) extra)))
 	     {
 	       /* this specific rule already exists */
 	       pRule = cur;
@@ -266,7 +417,8 @@ unsigned int        extra;
 	  }
 	
 	/* check for extra specification in case of function rule */
-	if ((type == PRFunction) && (cur->PrPresFunction != (FunctionType) extra))
+	if ((type == PRFunction) &&
+	    (cur->PrPresFunction != (FunctionType) extra))
 	  {
 	    prev = cur;
 	    cur = cur->PrNextPRule;
@@ -587,7 +739,7 @@ AttributePres     **attrblock;
   PtrPRule           *ppRule;
   PtrSSchema          pSS;
   AttributePres      *attrs;
-  STRING              attrVal;
+  CHAR_T*             attrVal;
   unsigned int        elementType;
   int                 nbrules;
   int                 i, j, val;
@@ -602,7 +754,7 @@ AttributePres     **attrblock;
   elementType = ctxt->name[att];
   for (i = 0; i < nbrules && !ppRule && attrs; i++)
     {
-      if ((att > 0 && attrs->ApElemType != ctxt->type) ||
+      if ((att > 0 && attrs->ApElemType != (int)(ctxt->type)) ||
 	  (att == 0 && attrs->ApElemType != 0))
 	{
 	  if (ctxt->type == 0)
@@ -832,7 +984,7 @@ unsigned int    att;
    i = 0;
    while (i < MAX_ANCESTORS && ctxt->name[i] != 0)
      {
-       if (ctxt->names_nb[i] > 0 && i != att)
+       if (ctxt->names_nb[i] > 0 && i != (int)att)
 	 {
 	   cond = firstCond;
 	   while (cond &&
@@ -918,7 +1070,8 @@ PtrPRule          **chain;
 	   pRule->PrPresFunction >  (FunctionType) extra))
 	  pRule = NULL;
       else if (pRule->PrType != pres ||
-	       (pres == PtFunction && pRule->PrPresFunction !=  (FunctionType) extra))
+	       (pres == PtFunction &&
+		pRule->PrPresFunction != (FunctionType) extra))
 	/* check for extra specification in case of function rule */
 	{
 	  *chain = &(pRule->PrNextPRule);
@@ -1096,11 +1249,16 @@ ThotBool            generic;
     {
     case STYLE_UNIT_REL:
       int_unit = UnRelative;
-      if (type == PtBreak1 ||
-	  type == PtBreak2 ||
-	  type == PtIndent ||
-	  type == PtLineSpacing ||
-	  type == PtLineWeight)
+      if (type == PtHeight ||            type == PtWidth ||
+	  type == PtMarginTop ||         type == PtMarginRight ||
+	  type == PtMarginBottom ||      type == PtMarginLeft ||
+	  type == PtPaddingTop ||        type == PtPaddingRight ||
+	  type == PtPaddingBottom ||     type == PtPaddingLeft ||
+	  type == PtBorderTopWidth ||    type == PtBorderRightWidth ||
+	  type == PtBorderBottomWidth || type == PtBorderLeftWidth ||
+          type == PtLineWeight ||              type == PtThickness ||
+	  type == PtIndent ||            type == PtLineSpacing ||
+           type == PtBreak1 ||            type == PtBreak2)
 	value *= 10;
       break;
     case STYLE_UNIT_EM:
@@ -1156,6 +1314,10 @@ ThotBool            generic;
     case PtFillPattern:
     case PtBackground:
     case PtForeground:
+    case PtBorderTopColor:
+    case PtBorderRightColor:
+    case PtBorderBottomColor:
+    case PtBorderLeftColor:
       rule->PrPresMode = PresImmediate;
       rule->PrAttrValue = 0;
       rule->PrIntValue = value;
@@ -1173,6 +1335,13 @@ ThotBool            generic;
 	case STYLE_FONT_COURIER:
 	  rule->PrChrValue = 'C';
 	  break;
+#   ifdef _WINDOWS
+#   ifdef _I18N_
+	case STYLE_FONT_OTHER:
+	  rule->PrChrValue = -1;
+	  break;
+#   endif /* _I18N_ */
+#   endif /* _WINDOWS */
 	}
       break;
     case PtStyle:
@@ -1226,11 +1395,61 @@ ThotBool            generic;
     case PtThickness:
     case PtLineStyle:
       break;
+    case PtBorderTopStyle:
+    case PtBorderRightStyle:
+    case PtBorderBottomStyle:
+    case PtBorderLeftStyle:
+      switch (value)
+	{
+	case STYLE_BORDERNONE:
+	  rule->PrChrValue = '0';
+	  break;
+	case STYLE_BORDERHIDDEN:
+	  rule->PrChrValue = 'H';
+	  break;
+	case STYLE_BORDERDOTTED:
+	  rule->PrChrValue = '.';
+	  break;
+	case STYLE_BORDERDASHED:
+	  rule->PrChrValue = '-';
+	  break;
+	case STYLE_BORDERSOLID:
+	  rule->PrChrValue = 'S';
+	  break;
+	case STYLE_BORDERDOUBLE:
+	  rule->PrChrValue = 'D';
+	  break;
+	case STYLE_BORDERGROOVE:
+	  rule->PrChrValue = 'G';
+	  break;
+	case STYLE_BORDERRIDGE:
+	  rule->PrChrValue = 'R';
+	  break;
+	case STYLE_BORDERINSET:
+	  rule->PrChrValue = 'I';
+	  break;
+	case STYLE_BORDEROUTSET:
+	  rule->PrChrValue = 'O';
+	  break;
+	}
+      break;
     case PtBreak1:
     case PtBreak2:
     case PtIndent:
     case PtLineSpacing:
     case PtLineWeight:
+    case PtMarginTop:
+    case PtMarginLeft:
+    case PtMarginBottom:
+    case PtMarginRight:
+    case PtBorderTopWidth:
+    case PtBorderLeftWidth:
+    case PtBorderBottomWidth:
+    case PtBorderRightWidth:
+    case PtPaddingTop:
+    case PtPaddingLeft:
+    case PtPaddingBottom:
+    case PtPaddingRight:
       rule->PrPresMode = PresImmediate;
       rule->PrMinUnit = int_unit;
       rule->PrMinValue = value;
@@ -1448,6 +1667,15 @@ ThotBool            generic;
 	      rule->PrPresBox[0] = RealSize;
 	    }
 	  break;
+	case FnPage:
+	  rule->PrPresFunction = (FunctionType) funcType;
+	  rule->PrPresBoxRepeat = FALSE;
+	  rule->PrExternal = FALSE;
+	  rule->PrElement = FALSE;
+	  rule->PrNPresBoxes = 0;
+	  rule->PrPresBox[0] = 0;
+	  rule->PrPresBoxName[0] = WC_EOS;
+	  break;
 	}
       break;
     }
@@ -1479,8 +1707,13 @@ PtrPRule                   rule;
     case PtFillPattern:
     case PtBackground:
     case PtForeground:
+    case PtBorderTopColor:
+    case PtBorderRightColor:
+    case PtBorderBottomColor:
+    case PtBorderLeftColor:
       value = rule->PrIntValue;
       break;
+
     case PtFont:
       switch (rule->PrChrValue)
 	{
@@ -1493,8 +1726,16 @@ PtrPRule                   rule;
 	case 'C':
 	  value = STYLE_FONT_COURIER;
 	  break;
+#   ifdef _WINDOWS
+#   ifdef _I18N_
+	case -1:
+	  value = STYLE_FONT_OTHER;
+	  break;
+#   endif /* _I18N_ */
+#   endif /* _WINDOWS */
 	}
       break;
+
     case PtStyle:
       switch (rule->PrChrValue)
 	{
@@ -1512,6 +1753,7 @@ PtrPRule                   rule;
 	  break;
 	}
       break;
+
     case PtWeight:
       switch (rule->PrChrValue)
 	{
@@ -1526,9 +1768,13 @@ PtrPRule                   rule;
 	  break;
 	}
       break;
+
     case PtUnderline:
       switch (rule->PrChrValue)
 	{
+	case 'N':
+	  value = STYLE_NOUNDERLINE;
+	  break;
 	case 'U':
 	  value = STYLE_UNDERLINE;
 	  break;
@@ -1540,17 +1786,94 @@ PtrPRule                   rule;
 	  break;
 	}
       break;
+
     case PtThickness:
-    case PtLineStyle:
+      switch (rule->PrChrValue)
+	{
+	case 'N':
+	  value = STYLE_THINUNDERLINE;
+	  break;
+	case 'T':
+	  value = STYLE_THICKUNDERLINE;
+	  break;
+	}
       break;
+
+    case PtLineStyle:
+      switch (rule->PrChrValue)
+	{
+	case 'S':
+	  value = STYLE_LINESOLID;
+	  break;
+	case '-':
+	  value = STYLE_LINEDASHED;
+	  break;
+	case '.':
+	  value = STYLE_LINEDOTTED;
+	  break;
+	}
+      break;
+
+    case PtBorderTopStyle:
+    case PtBorderRightStyle:
+    case PtBorderBottomStyle:
+    case PtBorderLeftStyle:
+      switch (rule->PrChrValue)
+	{
+	case '0':
+	  value = STYLE_BORDERNONE;
+	  break;
+	case 'H':
+	  value = STYLE_BORDERHIDDEN;
+	  break;
+	case '.':
+	  value = STYLE_BORDERDOTTED;
+	  break;
+	case '-':
+	  value = STYLE_BORDERDASHED;
+	  break;
+	case 'S':
+	  value = STYLE_BORDERSOLID;
+	  break;
+	case 'D':
+	  value = STYLE_BORDERDOUBLE;
+	  break;
+	case 'G':
+	  value = STYLE_BORDERGROOVE;
+	  break;
+	case 'R':
+	  value = STYLE_BORDERRIDGE;
+	  break;
+	case 'I':
+	  value = STYLE_BORDERINSET;
+	  break;
+	case 'O':
+	  value = STYLE_BORDEROUTSET;
+	  break;
+	}
+      break;
+
     case PtBreak1:
     case PtBreak2:
     case PtIndent:
     case PtLineSpacing:
     case PtLineWeight:
+    case PtMarginTop:
+    case PtMarginRight:
+    case PtMarginBottom:
+    case PtMarginLeft:
+    case PtPaddingTop:
+    case PtPaddingRight:
+    case PtPaddingBottom:
+    case PtPaddingLeft:
+    case PtBorderTopWidth:
+    case PtBorderRightWidth:
+    case PtBorderBottomWidth:
+    case PtBorderLeftWidth:
       int_unit = rule->PrMinUnit;
       value = rule->PrMinValue;
       break;
+
     case PtSize:
       if (rule->PrPresMode == PresInherit)
 	{
@@ -1563,12 +1886,43 @@ PtrPRule                   rule;
         value = rule->PrMinValue;
 	}
       break;
+
+    case PtVertRef:
+    case PtHorizRef:
+    case PtVertPos:
+    case PtHorizPos:
+      int_unit = rule->PrPosRule.PoDistUnit;
+      value = rule->PrPosRule.PoDistance;
+      break;
+
+    case PtHeight:
+    case PtWidth:
+      int_unit = rule->PrDimRule.DrUnit;
+      if (int_unit == 0)
+	int_unit = UnPixel;
+
+      value = rule->PrDimRule.DrValue;
+      break;
+
     case PtJustify:
       if (rule->PrJustify)
 	value = STYLE_JUSTIFIED;
       else
 	value = STYLE_NOTJUSTIFIED;
       break;
+
+    case PtHyphenate:
+      if (rule->PrJustify)
+	value = STYLE_HYPHENATE;
+      else
+	value = STYLE_NOHYPHENATE;
+      break;
+
+    case PtPictInfo:
+    case PtVertOverflow:
+    case PtHorizOverflow:
+      break;
+
     case PtAdjust:
       switch (rule->PrAdjust)
 	{
@@ -1589,31 +1943,7 @@ PtrPRule                   rule;
 	  break;
 	}
       break;
-    case PtHyphenate:
-      if (rule->PrJustify)
-	value = STYLE_HYPHENATE;
-      else
-	value = STYLE_NOHYPHENATE;
-      break;
-    case PtVertRef:
-    case PtHorizRef:
-    case PtVertPos:
-    case PtHorizPos:
-      int_unit = rule->PrPosRule.PoDistUnit;
-      value = rule->PrPosRule.PoDistance;
-      break;
-    case PtHeight:
-    case PtWidth:
-      int_unit = rule->PrDimRule.DrUnit;
-      if (int_unit == 0)
-	int_unit = UnPixel;
 
-      value = rule->PrDimRule.DrValue;
-      break;
-    case PtPictInfo:
-    case PtVertOverflow:
-    case PtHorizOverflow:
-      break;
     case PtFunction:
       switch (rule->PrPresFunction)
 	{
@@ -1655,10 +1985,14 @@ PtrPRule                   rule;
 	      value = STYLE_HREPEAT; break;
 	    case YRepeat:
 	      value = STYLE_VREPEAT; break;
+	      break;
 	    default:
 	      unit = STYLE_UNIT_INVALID;
 	      value = 0;
 	    }
+	case FnPage:
+	  value = STYLE_ALWAYS;
+	  unit = STYLE_UNIT_REL;
 	  break;
 	default:
 	  break;
@@ -1730,14 +2064,29 @@ ThotBool           *absolute;
     case PRVisibility:
       *intRule = PtVisibility;
       break;
-    case PRForeground:
-      *intRule = PtForeground;
+    case PRFillPattern:
+      *intRule = PtFillPattern;
       break;
     case PRBackground:
       *intRule = PtBackground;
       break;
-    case PRSize:
-      *intRule = PtSize;
+    case PRForeground:
+      *intRule = PtForeground;
+      break;
+    case PRBorderTopColor:
+      *intRule = PtBorderTopColor;
+      break;
+    case PRBorderRightColor:
+      *intRule = PtBorderRightColor;
+      break;
+    case PRBorderBottomColor:
+      *intRule = PtBorderBottomColor;
+      break;
+    case PRBorderLeftColor:
+      *intRule = PtBorderLeftColor;
+      break;
+    case PRFont:
+      *intRule = PtFont;
       break;
     case PRStyle:
       *intRule = PtStyle;
@@ -1745,61 +2094,94 @@ ThotBool           *absolute;
     case PRWeight:
       *intRule = PtWeight;
       break;
-    case PRAdjust:
-      *intRule = PtAdjust;
+    case PRUnderline:
+      *intRule = PtUnderline;
+      break;
+    case PRBorderTopStyle:
+      *intRule = PtBorderTopStyle;
+      break;
+    case PRBorderRightStyle:
+      *intRule = PtBorderRightStyle;
+      break;
+    case PRBorderBottomStyle:
+      *intRule = PtBorderBottomStyle;
+      break;
+    case PRBorderLeftStyle:
+      *intRule = PtBorderLeftStyle;
       break;
     case PRIndent:
       *intRule = PtIndent;
       break;
+    case PRSize:
+      *intRule = PtSize;
+      break;
+    case PRLineSpacing:
+      *intRule = PtLineSpacing;
+      break;
+    case PRMarginTop:
+      *intRule = PtMarginTop;
+      break;
+    case PRMarginRight:
+      *intRule = PtMarginRight;
+      break;
+    case PRMarginBottom:
+      *intRule = PtMarginBottom;
+      break;
+    case PRMarginLeft:
+      *intRule = PtMarginLeft;
+      break;
+    case PRPaddingTop:
+      *intRule = PtPaddingTop;
+      break;
+    case PRPaddingRight:
+      *intRule = PtPaddingRight;
+      break;
+    case PRPaddingBottom:
+      *intRule = PtPaddingBottom;
+      break;
+    case PRPaddingLeft:
+      *intRule = PtPaddingLeft;
+      break;
+    case PRBorderTopWidth:
+      *intRule = PtBorderTopWidth;
+      break;
+    case PRBorderRightWidth:
+      *intRule = PtBorderRightWidth;
+      break;
+    case PRBorderBottomWidth:
+      *intRule = PtBorderBottomWidth;
+      break;
+    case PRBorderLeftWidth:
+      *intRule = PtBorderLeftWidth;
+      break;      
+    case PRVertPos:
+      *intRule = PtVertPos;
+      break;
+    case PRHorizPos:
+      *intRule = PtHorizPos;
+      break;
+    case PRHeight:
+      *intRule = PtHeight;
+      *absolute = TRUE;
+      break;
+    case PRWidth:
+      *intRule = PtWidth;
+      *absolute = TRUE;
+      break;
     case PRJustify:
       *intRule = PtJustify;
       break;
     case PRHyphenate:
       *intRule = PtHyphenate;
       break;
-    case PRUnderline:
-      *intRule = PtUnderline;
-      break;
-    case PRFillPattern:
-      *intRule = PtFillPattern;
-      break;
-    case PRFont:
-      *intRule = PtFont;
-      break;
-    case PRLineSpacing:
-      *intRule = PtLineSpacing;
-      break;
-    case PRVertPos:
-      *intRule = PtVertPos;
-      break;
-    case PRHorizPos:
-      *intRule = PtHorizPos;
-      break;
-    case PRWidth:
-      *intRule = PtWidth;
-      *absolute = TRUE;
-      break;
-    case PRHeight:
-      *intRule = PtHeight;
-      *absolute = TRUE;
-      break;
-    case PRTMargin:
-      *intRule = PtVertPos;
-      break;
-    case PRLMargin:
-      *intRule = PtHorizPos;
-      break;
-    case PRBMargin:
-      *intRule = PtHeight;
-      break;
-    case PRRMargin:
-      *intRule = PtWidth;
+    case PRVertOverflow:
+      *intRule = PtVertOverflow;
       break;
     case PRHorizOverflow:
       *intRule = PtHorizOverflow;
       break;
-    case PRVertOverflow:
-      *intRule = PtVertOverflow;
+    case PRAdjust:
+      *intRule = PtAdjust;
       break;
     case PRShowBox:
       *intRule = PtFunction;
@@ -1821,6 +2203,10 @@ ThotBool           *absolute;
       *intRule = PtFunction;
       *func = FnLine;
       break;
+    case PRPageBefore:
+      *intRule = PtFunction;
+      *func = FnPage;
+      break;
     default:
       *intRule = PtFunction;
     }
@@ -1961,36 +2347,6 @@ int                 extra;
     case PtVisibility:
       setting->type = PRVisibility;
       break;
-    case PtFont:
-      setting->type = PRFont;
-      break;
-    case PtStyle:
-      setting->type = PRStyle;
-      break;
-    case PtWeight:
-      setting->type = PRWeight;
-      break;
-    case PtSize:
-      setting->type = PRSize;
-      break;
-    case PtUnderline:
-      setting->type = PRUnderline;
-      break;
-    case PtIndent:
-      setting->type = PRIndent;
-      break;
-    case PtLineSpacing:
-      setting->type = PRLineSpacing;
-      break;
-    case PtJustify:
-      setting->type = PRJustify;
-      break;
-    case PtAdjust:
-      setting->type = PRAdjust;
-      break;
-    case PtHyphenate:
-      setting->type = PRHyphenate;
-      break;
     case PtFillPattern:
       setting->type = PRFillPattern;
       break;
@@ -2000,11 +2356,86 @@ int                 extra;
     case PtForeground:
       setting->type = PRForeground;
       break;
-    case PtVertPos:
-      setting->type = PRTMargin;
+    case PtBorderTopColor:
+      setting->type = PRBorderTopColor;
       break;
-    case PtHorizPos:
-      setting->type = PRLMargin;
+    case PtBorderRightColor:
+      setting->type = PRBorderRightColor;
+      break;
+    case PtBorderBottomColor:
+      setting->type = PRBorderBottomColor;
+      break;
+    case PtBorderLeftColor:
+      setting->type = PRBorderLeftColor;
+      break;
+    case PtFont:
+      setting->type = PRFont;
+      break;
+    case PtStyle:
+      setting->type = PRStyle;
+      break;
+    case PtWeight:
+      setting->type = PRWeight;
+      break;
+    case PtUnderline:
+      setting->type = PRUnderline;
+      break;
+    case PtBorderTopStyle:
+      setting->type = PRBorderTopStyle;
+      break;
+    case PtBorderRightStyle:
+      setting->type = PRBorderRightStyle;
+      break;
+    case PtBorderBottomStyle:
+      setting->type = PRBorderBottomStyle;
+      break;
+    case PtBorderLeftStyle:
+      setting->type = PRBorderLeftStyle;
+      break;
+    case PtIndent:
+      setting->type = PRIndent;
+      break;
+    case PtSize:
+      setting->type = PRSize;
+      break;
+    case PtLineSpacing:
+      setting->type = PRLineSpacing;
+      break;
+    case PtMarginTop:
+      setting->type = PRMarginTop;
+      break;
+    case PtMarginRight:
+      setting->type = PRMarginRight;
+      break;
+    case PtMarginBottom:
+      setting->type = PRMarginBottom;
+      break;
+    case PtMarginLeft:
+      setting->type = PRMarginLeft;
+      break;
+    case PtPaddingTop:
+      setting->type = PRPaddingTop;
+      break;
+    case PtPaddingRight:
+      setting->type = PRPaddingRight;
+      break;
+    case PtPaddingBottom:
+      setting->type = PRPaddingBottom;
+      break;
+    case PtPaddingLeft:
+      setting->type = PRPaddingLeft;
+      break;
+    case PtBorderTopWidth:
+      setting->type = PRBorderTopWidth;
+      break;
+    case PtBorderRightWidth:
+      setting->type = PRBorderRightWidth;
+      break;
+    case PtBorderBottomWidth:
+      setting->type = PRBorderBottomWidth;
+      break;
+    case PtBorderLeftWidth:
+      setting->type = PRBorderLeftWidth;
       break;
     case PtHeight:
       setting->type = PRHeight;
@@ -2012,6 +2443,15 @@ int                 extra;
     case PtWidth:
       setting->type = PRWidth;
       break;
+    case PtJustify:
+      setting->type = PRJustify;
+      break;
+    case PtHyphenate:
+      setting->type = PRHyphenate;
+      break;
+    case PtAdjust:
+      setting->type = PRAdjust;
+      break;
     case PtFunction:
       switch (extra)
 	{
@@ -2027,6 +2467,9 @@ int                 extra;
 	case FnPictureMode:
 	  setting->type = PRPictureMode;
 	  break;
+	case FnPage:
+	  setting->type = PRPageBefore;
+	  break;
 	default:
 	  /* not yet supported by the driver */
 	  setting->type = PRNone;
@@ -2111,8 +2554,8 @@ PSchema             tsch;
 Document            doc;
 #endif
 {
-  PRule               rule;
-  PtrPRule            pRule = NULL;
+  PRule               rule, nextRule;
+  PtrPRule            pRule;
   PtrSSchema	      pSS;
   AttributePres      *attrs;
   DisplayMode         dispMode;
@@ -2126,7 +2569,7 @@ Document            doc;
   dispMode = TtaGetDisplayMode (doc);
   if (dispMode == DisplayImmediately)
     TtaSetDisplayMode (doc, DeferredDisplay);
-
+  pRule = NULL;
   if (el != NULL)
     {
       do
@@ -2134,8 +2577,10 @@ Document            doc;
 	  rule = (PRule) ((PtrElement) el)->ElFirstPRule;
 	  while (rule)
 	    {
+	      nextRule = rule;
+	      TtaNextPRule (el, &nextRule);
 	      TtaRemovePRule (el, rule, doc);
-	      TtaNextPRule (el, &rule);
+	      rule = nextRule;
 	    }
 	}
       while (rule != NULL);
