@@ -87,15 +87,16 @@ int                 button;
 
 #endif /* __STDC__ */
 {
-   int                 index;
    PtrBox              pBox;
    PtrTextBuffer       pBuffer;
    PtrAbstractBox      pAb;
    NotifyElement       notifyEl;
    PtrElement          el;
+   ViewFrame          *pFrame;
    int                 charsNumber;
    int                 spacesNumber;
-   ViewFrame          *pFrame;
+   int                 index, pos;
+   int                 xOrg, yOrg;
 
    if (frame >= 1)
      {
@@ -121,8 +122,8 @@ int                 button;
 	     if (pAb->AbLeafType == LtText &&
 		 (!pAb->AbPresentationBox || pAb->AbCanBeModified))
 	       {
-		  x -= pBox->BxXOrg;
-		  LocateClickedChar (pBox, &pBuffer, &x, &index, &charsNumber, &spacesNumber);
+		  pos = x - pBox->BxXOrg;
+		  LocateClickedChar (pBox, &pBuffer, &pos, &index, &charsNumber, &spacesNumber);
 		  charsNumber = pBox->BxIndChar + charsNumber + 1;
 	       }
 	  }
@@ -143,20 +144,27 @@ int                 button;
 	      ChangeSelection (frame, pAb, charsNumber, TRUE, TRUE, FALSE, TRUE);
           else /* button == 4 */
 	    {
-	      /* send event TteElemActivate.Pre to the application */
-	      el = pAb->AbElement;
-	      notifyEl.event = TteElemClick;
-	      notifyEl.document = FrameTable[frame].FrDoc;
-	      notifyEl.element = (Element) el;
-	      notifyEl.elementType.ElTypeNum = el->ElTypeNumber;
-	      notifyEl.elementType.ElSSchema = (SSchema) (el->ElStructSchema);
-	      notifyEl.position = 0;
-	      if (CallEventType ((NotifyEvent *) & notifyEl, TRUE))
-		/* the application asks Thot to do nothing */
-		return;
-	      /* send event TteElemActivate.Pre to the application */
-	      CallEventType ((NotifyEvent *) & notifyEl, FALSE);
-            }
+	      /* check if the curseur is within the box */
+	      xOrg =  pBox->BxXOrg + pBox->BxLMargin + pBox->BxLBorder + pBox->BxLPadding;
+	      yOrg =  pBox->BxYOrg + pBox->BxTMargin + pBox->BxTBorder + pBox->BxTPadding;
+	      if (x >= xOrg && x <= xOrg + pBox->BxW &&
+		  y >= yOrg && y <= yOrg + pBox->BxH)
+		{
+		  /* send event TteElemActivate.Pre to the application */
+		  el = pAb->AbElement;
+		  notifyEl.event = TteElemClick;
+		  notifyEl.document = FrameTable[frame].FrDoc;
+		  notifyEl.element = (Element) el;
+		  notifyEl.elementType.ElTypeNum = el->ElTypeNumber;
+		  notifyEl.elementType.ElSSchema = (SSchema) (el->ElStructSchema);
+		  notifyEl.position = 0;
+		  if (CallEventType ((NotifyEvent *) & notifyEl, TRUE))
+		    /* the application asks Thot to do nothing */
+		    return;
+		  /* send event TteElemActivate.Pre to the application */
+		  CallEventType ((NotifyEvent *) & notifyEl, FALSE);
+		}
+	    }
      }
 }
 
@@ -925,9 +933,14 @@ int                *pointselect;
 	       }
 	     return (NULL);
 	  }
-	/* C'est une boite eclatee ? */
 	else if (pBox->BxType == BoGhost)
+	  /* dummy box */
 	   return (NULL);
+	else if (pAb->AbLeafType == LtGraphics &&
+		 pAb->AbPresentationBox &&
+		 pAb->AbShape == '0')
+	  /* it's also a dummy box */
+	  return (NULL);
 	/* Si le pave englobe le point designe */
 	else if (pBox->BxXOrg <= lowerX
 		 && pBox->BxXOrg + pBox->BxWidth >= higherX
@@ -1027,7 +1040,8 @@ int                 yDelta;
 				     pBox->BxAbstractBox->AbElement->ElTypeNumber,
 				     pBox->BxAbstractBox->AbElement->ElStructSchema))
 	    pBox = pSourceBox;
-	  if (pBox == pSourceBox || pBox->BxAbstractBox->AbBox == pSourceBox)
+	  if (pBox == pSourceBox || pBox->BxAbstractBox->AbBox == pSourceBox ||
+	      IsParentBox (pSourceBox, pBox))
 	    {
 	      /* compute the height to be moved */
 	      pLine = SearchLine (pBox);
@@ -1054,6 +1068,7 @@ int                 yDelta;
 		  found = FALSE;
 		}
 	    }
+#ifdef IV
 	  else if (IsParentBox (pSourceBox, pBox))
 	    {
 	      if (xDelta > 0)
@@ -1085,6 +1100,7 @@ int                 yDelta;
 		    }
 		}
 	    }
+#endif
 	  else if (pBox->BxAbstractBox->AbLeafType != LtText && pBox->BxNChars != 0)
 	    {
 	      /* the box doesn't match, skip over */
@@ -1656,10 +1672,9 @@ ThotBool            pre;
 }
 
 /*----------------------------------------------------------------------
-   ApplyDirectTranslate recherche la boite selectionnee pour un changement 
-   de position. Si la plus petite boite englobant le point 
-   xm,ym de la fenetre frame ne peut pas etre deplacee, la 
-   procedure prend la boite englobante et ainsi de suite.  
+  ApplyDirectTranslate looks for the selected box for a move.
+  If the smaller box enclosing the point xm, ym of the window cannot
+  be moved, the function checks the encolsing box, etc.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
 void                ApplyDirectTranslate (int frame, int xm, int ym)
@@ -1762,9 +1777,8 @@ int                 ym;
 		       /* Note si le trace est ouvert ou ferme */
 		       still = (pAb->AbPolyLineShape == 'p' ||
 				pAb->AbPolyLineShape == 's');
-		       /* Reaffiche la selection */
-		       if (ThotLocalActions[T_switchsel])
-			 (*ThotLocalActions[T_switchsel]) (frame, FALSE);
+		       /* if (ThotLocalActions[T_switchsel])
+		       (*ThotLocalActions[T_switchsel]) (frame, FALSE);*/
 		       draw = GetParentDraw (pBox);
 		       PolyLineModification (frame, &x, &y, pBox, draw,
 					   pBox->BxNChars, pointselect, still);
@@ -1783,12 +1797,11 @@ int                 ym;
 		       if (width != pBox->BxWidth || height != pBox->BxHeight)
 			 NewDimension (pAb, width, height, frame, TRUE);
 #endif
-		       RedrawFrameBottom (frame, 0);
+		       RedrawFrameBottom (frame, 0, NULL);
 		       NewContent (pAb);
 		       APPgraphicModify (pEl, pointselect, frame, FALSE);
-		       /* Reaffiche la selection */
-		       if (ThotLocalActions[T_switchsel])
-			 (*ThotLocalActions[T_switchsel]) (frame, TRUE);
+		       /* if (ThotLocalActions[T_switchsel])
+			  (*ThotLocalActions[T_switchsel]) (frame, TRUE);*/
 		    }
 	       }
 	     else
@@ -2374,7 +2387,7 @@ int                *spacesNumber;
 	/* Calcule la largeur des blancs */
 	if (pBox->BxSpaceWidth == 0)
 	  {
-	     spaceWidth = CharacterWidth (_SPACE_, font);
+	     spaceWidth = CharacterWidth (SPACE, font);
 	     extraSpace = 0;
 	  }
 	else
@@ -2391,7 +2404,7 @@ int                *spacesNumber;
 	c = (UCHAR_T) ((*pBuffer)->BuContent[newIndex - 1]);
 	if (c == 0)
 	   charWidth = 0;
-	else if (c == _SPACE_)
+	else if (c == SPACE)
 	   charWidth = spaceWidth;
 	else
 	   charWidth = CharacterWidth (c, font);
@@ -2404,13 +2417,13 @@ int                *spacesNumber;
 	     c = (UCHAR_T) ((*pBuffer)->BuContent[newIndex - 1]);
 	     if (c == 0)
 		charWidth = 0;
-	     else if (c == _SPACE_)
+	     else if (c == SPACE)
 		charWidth = spaceWidth;
 	     else
 		charWidth = CharacterWidth (c, font);
 #endif
 
-	     if (c == _SPACE_)
+	     if (c == SPACE)
 	       {
 		  (*spacesNumber)++;
 		  if (extraSpace > 0)
@@ -2445,7 +2458,7 @@ int                *spacesNumber;
 	     c = (UCHAR_T) ((*pBuffer)->BuContent[newIndex - 1]);
 	     if (c == 0)
 		charWidth = 0;
-	     else if (c == _SPACE_)
+	     else if (c == SPACE)
 		charWidth = spaceWidth;
 	     else
 		charWidth = CharacterWidth (c, font);
@@ -2462,7 +2475,7 @@ int                *spacesNumber;
 #ifdef STRUCT_EDIT
 	     /* BAlignment sur le caractere */
 	     *x = dx - charWidth;
-	     if (c == _SPACE_)
+	     if (c == SPACE)
 	       {
 		  if (*spacesNumber > 0 && pBox->BxNPixels >= *spacesNumber)
 		     (*x)--;
