@@ -40,7 +40,6 @@ static ThotColorStruct  cwhite;
 static ThotPoint   *points;	/* control points for curbs */
 static int          npoints;
 static int          MAX_points;
-static int i;
 
 typedef struct stack_point
   {
@@ -64,14 +63,13 @@ static int          stack_deep;
   accordingly to the ascent of the font used.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                FontOrig (ptrfont font, CHAR_T firstchar, int *pX, int *pY)
+void          FontOrig (ptrfont font, CHAR_T firstchar, int *pX, int *pY)
 #else  /* __STDC__ */
-void                FontOrig (font, firstchar, pX, pY)
-ptrfont             font;
-CHAR_T                firstchar;
-int                *pX;
-int                *pY;
-
+void          FontOrig (font, firstchar, pX, pY)
+ptrfont       font;
+CHAR_T        firstchar;
+int          *pX;
+int          *pY;
 #endif /* __STDC__ */
 {
    if (!font)
@@ -159,29 +157,36 @@ int                 fg;
 
 #endif /* __STDC__ */
 {
-    CHAR_T                dash[2];
-    if (style == 0)
-      {
+  char              dash[2];
+
+  if (style >= 5)
+    {
+      /* solid */
 #ifdef _GTK
-	gdk_gc_set_line_attributes (TtLineGC, thick, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);
+      gdk_gc_set_line_attributes (TtLineGC, thick, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);
 #else /* _GTK */
       XSetLineAttributes (TtDisplay, TtLineGC, thick, LineSolid, CapButt, JoinMiter);
 #endif /* _GTK */
-     }
-    else
-      {
-	dash[0] = (CHAR_T) (style * 4);
-	dash[1] = (CHAR_T) 4;
+    }
+  else
+    {
+      if (style == 3)
+	/* dotted */
+	dash[0] = 4;
+      else
+	/* dashed */
+	dash[0] = 8;
+      dash[1] = 4;
 #ifdef _GTK
-	gdk_gc_set_dashes ( TtLineGC, 0, dash, 2); 
-	gdk_gc_set_line_attributes (TtLineGC, thick, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_MITER);
+      gdk_gc_set_dashes ( TtLineGC, 0, dash, 2); 
+      gdk_gc_set_line_attributes (TtLineGC, thick, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_MITER);
 #else /* _GTK */
-	XSetDashes (TtDisplay, TtLineGC, 0, dash, 2);
-	XSetLineAttributes (TtDisplay, TtLineGC, thick, LineOnOffDash, CapButt, JoinMiter);
+      XSetDashes (TtDisplay, TtLineGC, 0, dash, 2);
+      XSetLineAttributes (TtDisplay, TtLineGC, thick, LineOnOffDash, CapButt, JoinMiter);
 #endif /* _GTK */
-      }
-   /* Load the correct color */
-    LoadColor (disp, RO, active, fg);
+    }
+  /* Load the correct color */
+  LoadColor (disp, RO, active, fg);
 }
 
 
@@ -266,7 +271,7 @@ USTRING             text;
 		 case UNBREAKABLE_SPACE:
 		    text[i] = (UCHAR_T) SHOWN_UNBREAKABLE_SPACE;
 		    break;
-		 case _SPACE_:
+		 case SPACE:
 		    text[i] = (UCHAR_T) SHOWN_SPACE;
 		    break;
 	      }
@@ -514,7 +519,7 @@ int                 fg;
 	 *         bottom = y + ascent + 3;
 	 */
 
-	InitDrawing (0, 0, thickness, RO, active, fg);
+	InitDrawing (0, 5, thickness, RO, active, fg);
 	switch (type)
 	      {
 		 case 1:
@@ -564,7 +569,7 @@ int                 fg;
    int                 xcour, width, nb;
    STRING              ptcar;
 
-   font = ThotLoadFont (TEXT('L'), TEXT('t'), 0, 6, UnPoint, frame);
+   font = ThotLoadFont ('L', 't', 0, 6, UnPoint, frame);
    if (lgboite > 0)
      {
 	w = FrRef[frame];
@@ -626,11 +631,11 @@ int                 fg;
    xm = x + (fh / 2);
    xp = x + (fh / 4);
    y += FrameTable[frame].FrTopMargin;
-   InitDrawing (0, 0, 0, RO, active, fg);
+   InitDrawing (0, 5, 0, RO, active, fg);
    /* vertical part */
    DoDrawOneLine (frame, x, y + (2 * (h / 3)), xp - (thick / 2), y + h);
 
-   InitDrawing (0, 0, thick, RO, active, fg);
+   InitDrawing (0, 5, thick, RO, active, fg);
    /* Acending part */
    DoDrawOneLine (frame, xp, y + h, xm, y);
    /* Upper part */
@@ -778,12 +783,12 @@ int                 fg;
         y += FrameTable[frame].FrTopMargin;
 	xm = x + (l / 3);
 	ym = y + (h / 2) - 1;
-	InitDrawing (0, 0, 0, RO, active, fg);
+	InitDrawing (0, 5, 0, RO, active, fg);
 	/* Center */
 	DoDrawOneLine (frame, x, y + 1, xm, ym);
 	DoDrawOneLine (frame, x, y + h - 2, xm, ym);
 
-	InitDrawing (0, 0, 2, RO, active, fg);
+	InitDrawing (0, 5, 2, RO, active, fg);
 	/* Borders */
 	DoDrawOneLine (frame, x, y, x + l, y);
 	DoDrawOneLine (frame, x, y + h - 2, x + l, y + h - 2);
@@ -823,12 +828,12 @@ int                 fg;
    else
      {
         y += FrameTable[frame].FrTopMargin;
-	InitDrawing (0, 0, 0, RO, active, fg);
+	InitDrawing (0, 5, 0, RO, active, fg);
 	/* Vertical part */
 	DoDrawOneLine (frame, x + 2, y + 1, x + 2, y + h);
 	DoDrawOneLine (frame, x + l - 3, y + 1, x + l - 3, y + h);
 
-	InitDrawing (0, 0, 2, RO, active, fg);
+	InitDrawing (0, 5, 2, RO, active, fg);
 	/* Upper part */
 	DoDrawOneLine (frame, x + 1, y + 1, x + l, y);
 	FinishDrawing (0, RO, active);
@@ -869,7 +874,7 @@ int                 fg;
         y += FrameTable[frame].FrTopMargin;
 	/* radius of arcs is 6mm */
 	arc = h / 4;
-	InitDrawing (0, 0, 2, RO, active, fg);
+	InitDrawing (0, 5, 2, RO, active, fg);
 	/* vertical part */
 	DoDrawOneLine (frame, x + 1, y + arc, x + 1, y + h);
 	DoDrawOneLine (frame, x + l - 2, y + arc, x + l - 2, y + h);
@@ -918,7 +923,7 @@ int                 fg;
         y += FrameTable[frame].FrTopMargin;
 	/* radius of arcs is 3mm */
 	arc = h / 4;
-	InitDrawing (0, 0, 2, RO, active, fg);
+	InitDrawing (0, 5, 2, RO, active, fg);
 	/* two vertical lines */
 	DoDrawOneLine (frame, x + 1, y, x + 1, y + h - arc);
 	DoDrawOneLine (frame, x + l - 2, y, x + l - 2, y + h - arc);
@@ -1445,48 +1450,47 @@ int                 pattern;
 #endif /* __STDC__ */
 
 {
-   Pixmap              pat;
+  Pixmap              pat;
 
-   if (width <= 0 || height <= 0)
-      return;
+  if (width <= 0 || height <= 0)
+    return;
+  if (thick == 0 && pattern == 0)
+    return;
 
-   if (width > thick + 1)
-     width = width - thick - 1;
-   if (height > thick + 1)
-     height = height - thick - 1;
-   x += thick / 2;
-   y = y + thick / 2 + FrameTable[frame].FrTopMargin;
-
-   pat = (Pixmap) CreatePattern (0, RO, active, fg, bg, pattern);
-
-   if (pat != 0) {
+  y += FrameTable[frame].FrTopMargin;
+  pat = (Pixmap) CreatePattern (0, RO, active, fg, bg, pattern);
+  if (pat != 0)
+    {
 #ifdef _GTK
-     gdk_gc_set_tile ( TtGreyGC, pat);    
-     gdk_draw_rectangle ( FrRef[frame], TtGreyGC, TRUE, 
-			  x, y, width-1, height-1);
-     gdk_pixmap_unref (pat);
+      gdk_gc_set_tile ( TtGreyGC, pat);    
+      gdk_draw_rectangle ( FrRef[frame], TtGreyGC, TRUE, x, y, width, height);
+      gdk_pixmap_unref (pat);
 #else /* _GTK */
       XSetTile (TtDisplay, TtGreyGC, pat);
-    
-      XFillRectangle (TtDisplay, FrRef[frame], TtGreyGC,
-                      x, y, width, height);
+      XFillRectangle (TtDisplay, FrRef[frame], TtGreyGC, x, y, width, height);
       XFreePixmap (TtDisplay, pat);
 #endif /* _GTK */
-   }
+    }
 
-   /* Draw the border */
-   if (thick > 0)
-     {
-	InitDrawing (0, style, thick, RO, active, fg); 
+  /* Draw the border */
+  if (thick > 0)
+    {
+      if (width > thick)
+	width = width - thick;
+      if (height > thick)
+	height = height - thick;
+      x = x + thick / 2;
+      y = y + thick / 2;
+
+      InitDrawing (0, style, thick, RO, active, fg); 
 #ifdef _GTK
-	gdk_draw_rectangle ( FrRef[frame],TtLineGC,FALSE, 
-			     x, y, width, height);
+      gdk_draw_rectangle ( FrRef[frame],TtLineGC,FALSE, x, y, width, height);
 
 #else /* _GTK */
-	XDrawRectangle (TtDisplay, FrRef[frame], TtLineGC, x, y, width, height);
+      XDrawRectangle (TtDisplay, FrRef[frame], TtLineGC, x, y, width, height);
 #endif /* _GTK */
-	FinishDrawing (0, RO, active);
-   }
+      FinishDrawing (0, RO, active);
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -1601,8 +1605,11 @@ int                 arrow;
 #endif /* __STDC__ */
 
 {
+#ifdef _GTK
+   int                 k;
+#endif /* _GTK */
    ThotPoint          *points;
-   int                 i, j, k;
+   int                 i, j;
    PtrTextBuffer       adbuff;
 
    if (thick == 0)
@@ -2170,6 +2177,9 @@ int                 pattern;
 #endif /* __STDC__ */
 
 {
+#ifdef _GTK
+   int                 i;
+#endif /* _GTK */
    Pixmap              pat;
    int                 arc;
    int                 xf, yf;
@@ -2385,11 +2395,11 @@ int                 pattern;
 }
 
 /*----------------------------------------------------------------------
-  DrawVerticalLine draw a vertical line aligned top center or bottom
+  DrawVerticalLine draw a horizontal line aligned top center or bottom
   depending on align value.
-  RO indicates whether it's a read-only box
-  active indicates if the box is active
-  parameter fg indicates the drawing color
+  RO indicates whether it's a read-only box.
+  active indicates if the box is active.
+  The parameter fg indicates the drawing color.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
 void                DrawHorizontalLine (int frame, int thick, int style, int x, int y, int l, int h, int align, int RO, int active, int fg)
@@ -2411,21 +2421,21 @@ int                 fg;
 #endif /* __STDC__ */
 
 {
-   register int        Y;
+  int        Y;
 
-   y += FrameTable[frame].FrTopMargin;
-   if (align == 1)
-      Y = y + (h - thick) / 2;
-   else if (align == 2)
-      Y = y + h - thick - 1;
-   else
-      Y = y;
-   if (thick > 0)
-     {
-	InitDrawing (0, style, thick, RO, active, fg);
-	DoDrawOneLine (frame, x, Y, x + l - 1, Y);
-	FinishDrawing (0, RO, active);
-     }
+  y += FrameTable[frame].FrTopMargin;
+  if (align == 1)
+    Y = y + (h - thick) / 2;
+  else if (align == 2)
+    Y = y + h - (thick + 1) / 2;
+  else
+    Y = y + thick / 2;
+  if (thick > 0)
+    {
+      InitDrawing (0, style, thick, RO, active, fg);
+      DoDrawOneLine (frame, x, Y, x + l, Y);
+      FinishDrawing (0, RO, active);
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -2455,22 +2465,22 @@ int                 fg;
 #endif /* __STDC__ */
 
 {
-   register int        X;
+  int        X;
 
-   if (align == 1)
-      X = x + (l - thick) / 2;
-   else if (align == 2)
-      X = x + l - thick;
-   else
-      X = x;
+  if (align == 1)
+    X = x + (l - thick) / 2;
+  else if (align == 2)
+    X = x + l - (thick + 1) / 2;
+  else
+    X = x + thick / 2;
 
-   y += FrameTable[frame].FrTopMargin;
-   if (thick > 0)
-     {
-	InitDrawing (0, style, thick, RO, active, fg);
-	DoDrawOneLine (frame, X, y, X, y + h);
-	FinishDrawing (0, RO, active);
-     }
+  y += FrameTable[frame].FrTopMargin;
+  if (thick > 0)
+    {
+      InitDrawing (0, style, thick, RO, active, fg);
+      DoDrawOneLine (frame, X, y, X, y + h);
+      FinishDrawing (0, RO, active);
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -2621,6 +2631,9 @@ int                 bg;
 int                 pattern;
 #endif /* __STDC__ */
 {
+#ifdef _GTK
+   int                 i;
+#endif /* _GTK */
    int                 arc, arc2, xf, yf;
    XArc                xarc[4];
    XSegment            seg[5];
@@ -2745,12 +2758,13 @@ int                 pattern;
 	gdk_gc_set_tile (TtGreyGC, pat);
 	gdk_draw_polygon (FrRef[frame], TtGreyGC, TRUE,  point, 13);
 	/* Trace quatre arcs de cercle */
-	for (i=0;i<4;i++){  
+	for (i = 0; i < 4; i++)
+	  {  
 	  gdk_draw_arc (FrRef[frame], TtGreyGC,TRUE,  
 			xarc[i].x, xarc[i].y, 
 			xarc[i].width, xarc[i].height, 
-			xarc[i].angle1,xarc[i].angle2);
-	}
+			xarc[i].angle1, xarc[i].angle2);
+	  }
 	gdk_pixmap_unref (pat);
 #else /* _GTK */
 	XSetTile (TtDisplay, TtGreyGC, pat);
@@ -2767,28 +2781,32 @@ int                 pattern;
      {
 	InitDrawing (0, style, thick, RO, active, fg);
 #ifdef _GTK
-	for (i=0;i<4;i++){  
+	for (i = 0 ; i < 4; i++)
+	  {  
 	  gdk_draw_arc (FrRef[frame], TtLineGC, FALSE, 
 			xarc[i].x, xarc[i].y, 
 			xarc[i].width, xarc[i].height, 
-			xarc[i].angle1,xarc[i].angle2); 
-	}
+			xarc[i].angle1, xarc[i].angle2); 
+	  }
 #else /* _GTK */
 	XDrawArcs (TtDisplay, FrRef[frame], TtLineGC, xarc, 4);
 #endif /* _GTK */
-	if (arc2 < height / 2) {
+	if (arc2 < height / 2)
+	  {
 #ifdef _GTK
 	   gdk_draw_segments (FrRef[frame], TtLineGC, seg, 5);
 #else /* _GTK */
 	   XDrawSegments (TtDisplay, FrRef[frame], TtLineGC, seg, 5);
 #endif /* _GTK */
-	} else {
+	  }
+	else
+	  {
 #ifdef _GTK
 	   gdk_draw_segments (FrRef[frame], TtLineGC, seg, 4);
 #else /* _GTK */
 	   XDrawSegments (TtDisplay, FrRef[frame], TtLineGC, seg, 4);
 #endif /* _GTK */
-	}
+	  }
 	FinishDrawing (0, RO, active);
      }
 }
@@ -3029,62 +3047,25 @@ int xf;
 int yf;
 #endif /* __STDC__ */
 {
-   if (FrRef[frame] != None) {
+  if (FrRef[frame] != None)
+    {
 #ifdef _GTK
       gdk_window_copy_area (FrRef[frame], TtWhiteGC,
-			   xf,
-			   yf + FrameTable[frame].FrTopMargin,
-			   FrRef[frame],
-			   xd,
-			   yd + FrameTable[frame].FrTopMargin,
-			   width,
-			   height);
+			    xf,
+			    yf + FrameTable[frame].FrTopMargin,
+			    FrRef[frame],
+			    xd,
+			    yd + FrameTable[frame].FrTopMargin,
+			    width,
+			    height);
 #else /* _GTK */
       XCopyArea (TtDisplay, FrRef[frame], FrRef[frame], TtWhiteGC,
 		 xd, yd + FrameTable[frame].FrTopMargin, width, height,
 		 xf, yf + FrameTable[frame].FrTopMargin);
 #endif /* _GTK */
-   }
+    }
 }
 
-/*----------------------------------------------------------------------
-  EndOfString check wether string end by suffix.
-
-  ----------------------------------------------------------------------*/
-#ifdef __STDC__
-int                 EndOfString (STRING string, STRING suffix)
-#else  /* __STDC__ */
-int                 EndOfString (string, suffix)
-STRING              string;
-STRING              suffix;
-
-#endif /* __STDC__ */
-{
-   int                 string_lenght, suffix_lenght;
-
-   string_lenght = ustrlen (string);
-   suffix_lenght = ustrlen (suffix);
-   if (string_lenght < suffix_lenght)
-      return 0;
-   else
-      return (ustrcmp (string + string_lenght - suffix_lenght, suffix) == 0);
-}
-
-/*----------------------------------------------------------------------
-  XFlushOutput enforce updating of the calculated image for frame.
-  ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                XFlushOutput (int frame)
-#else  /* __STDC__ */
-void                XFlushOutput (frame)
-int                 frame;
-
-#endif /* __STDC__ */
-{
-#ifndef _GTK
-   XFlush (TtDisplay);
-#endif /* _GTK */
-}
 
 /*----------------------------------------------------------------------
   PaintWithPattern fill the rectangle associated to a window w (or frame if w= 0)
